import logging
import os
import shutil
from abc import ABC, abstractmethod

logger = logging.getLogger(__name__)


class Benchmark(ABC):
    def __init__(self, benchmark_name: str, benchmark_dir: str):
        self.benchmark_name = benchmark_name
        self.benchmark_dir = benchmark_dir
        self.metrics_dir = os.path.join(self.benchmark_dir, "results")
        self.benchmark_dir_s3 = None

    @abstractmethod
    def run(self):
        raise NotImplementedError

    def upload_metrics(self, s3_bucket: str, s3_dir: str):
        """Uploads benchmark metrics to an S3 bucket.

        Args:
            s3_bucket (str): The name of the S3 bucket to upload the metrics to.
            s3_dir (str): The S3 path of the directory to upload the metrics to.

        Returns:
            None
        """

        import boto3

        logging.info("Saving metrics to S3 Bucket %s...", s3_bucket)
        self.benchmark_dir_s3 = f"s3://{s3_bucket}/{s3_dir}"
        s3 = boto3.client("s3")

        if len(os.listdir(self.metrics_dir)) == 0:
            logger.warning("No metrics were created.")
            return

        for root, dirs, files in os.walk(self.metrics_dir):
            for filename in files:
                local_path = os.path.join(root, filename)

                relative_path = os.path.relpath(local_path, self.metrics_dir)
                s3_path = os.path.join(s3_dir, relative_path)

                # upload the file to S3
                s3.upload_file(local_path, s3_bucket, s3_path)

        logging.info("Metrics under %s have been saved to s3://%s/%s.", self.metrics_dir, s3_bucket, s3_dir)

    def cleanup_metrics(self):
        """
        Remove benchmark metrics from local and S3 storage.

        This method removes the directory specified by `self.benchmark_dir` from the local file system.
        If `self.benchmark_dir_s3` is also specified, it removes the corresponding directory from S3.

        """
        shutil.rmtree(self.benchmark_dir)
        if self.benchmark_dir_s3 is not None:
            import boto3

            s3 = boto3.resource("s3")
            bucket_name = self.benchmark_dir_s3.split("//")[-1].split("/")[0]
            benchmark_dir = self.benchmark_dir_s3.split(bucket_name)[-1].lstrip("/")
            bucket = s3.Bucket(bucket_name)
            bucket.objects.filter(Prefix=benchmark_dir).delete()
            s3.Object(bucket_name, self.benchmark_dir_s3).delete()



import datetime
import os
import unittest
from unittest.mock import ANY, MagicMock, call, patch

import pandas as pd
import yaml

from autogluon.bench.eval.hardware_metrics import hardware_metrics
from autogluon.bench.eval.hardware_metrics.hardware_metrics import (
    get_hardware_metrics,
    get_instance_id,
    get_instance_util,
    get_job_ids,
    get_metrics,
)

test_dir = os.path.dirname(__file__)
config_file = os.path.join(test_dir, "resources/test_config.yaml")
if not config_file:
    raise ValueError("Invalid Config File")
with open(config_file, "r") as f:
    config = yaml.safe_load(f)

hardware_metrics.aws_account_id = config["CDK_DEPLOY_ACCOUNT"]
hardware_metrics.aws_account_region = config["CDK_DEPLOY_REGION"]

from resources.expected_metrics import metrics

mock_cloudwatch_response = {
    "Label": "CPUUtilization",
    "Datapoints": [
        {"Timestamp": datetime.datetime(2023, 7, 12, 17, 39), "Average": 11.472356376239336, "Unit": "Percent"}
    ],
    "ResponseMetadata": {
        "RequestId": "93ed0de6-7f3c-4af8-8650-2310042c97f8",
        "HTTPStatusCode": 200,
        "HTTPHeaders": {
            "x-amzn-requestid": "93ed0de6-7f3c-4af8-8650-2310042c97f8",
            "content-type": "text/xml",
            "content-length": "512",
            "date": "Wed, 12 Jul 2023 18:19:56 GMT",
        },
        "RetryAttempts": 0,
    },
}

mock_results_df = pd.read_csv(os.path.join(test_dir, "resources/results.csv"))


class TestHardwareMetrics(unittest.TestCase):
    def test_get_job_ids(self):
        job_ids = get_job_ids(config)
        self.assertEqual(job_ids, ["123456-abc-efg", "010101-xxx-zzz"])

    @patch("boto3.client")
    def test_get_instance_id(self, mock_client):
        job_ids = get_job_ids(config)
        batch_client, ecs_client = MagicMock(), MagicMock()
        mock_client.side_effect = [batch_client, ecs_client]
        mock_batch_response = {
            "jobs": [
                {
                    "container": {
                        "containerInstanceArn": "abc",
                        "taskArn": "arn:aws:ecs:us-east-2:123456789:task/agbenchcomputeenvironmen-DhbZ6yaLr_Batch/b3bb44aa78f",
                    }
                }
            ]
        }
        batch_client.describe_jobs.return_value = mock_batch_response

        mock_ecs_response = {"containerInstances": [{"ec2InstanceId": 12345}]}
        ecs_client.describe_container_instances.return_value = mock_ecs_response
        instance_id = get_instance_id(job_ids[0])
        self.assertEqual(instance_id, 12345)
        cluster = f"arn:aws:ecs:{hardware_metrics.aws_account_region}:{hardware_metrics.aws_account_id}:cluster/agbenchcomputeenvironmen-DhbZ6yaLr_Batch"
        ecs_client.describe_container_instances.assert_called_once_with(cluster=cluster, containerInstances=["abc"])

    @patch("boto3.client")
    def test_get_instance_util(self, mock_client):
        cloudwatch_client = MagicMock()
        mock_client.side_effect = [cloudwatch_client]
        cloudwatch_client.get_metric_statistics.return_value = mock_cloudwatch_response
        self.assertEqual(
            get_instance_util(
                namespace="namespace",
                instance_id="1234",
                metric="CPUUtilization",
                start_time=datetime.datetime(2023, 7, 12, 17, 39),
                end_time=datetime.datetime(2023, 7, 12, 16, 39),
                cloudwatch_client=cloudwatch_client,
            ),
            cloudwatch_client.get_metric_statistics.return_value,
        )

    @patch("pandas.read_csv")
    @patch("autogluon.bench.eval.hardware_metrics.hardware_metrics.get_instance_id")
    @patch("autogluon.bench.eval.hardware_metrics.hardware_metrics.get_instance_util")
    @patch("autogluon.bench.eval.hardware_metrics.hardware_metrics.find_s3_file")
    def test_get_metrics(self, mock_s3_file, mock_instance_util, mock_instance_id, mock_csv):
        mock_csv.return_value = mock_results_df
        mock_instance_id.return_value = "12345"
        job_id = list(config.get("job_configs", {}).keys())[0]
        mock_instance_util.return_value = mock_cloudwatch_response
        mock_list_metrics_return = {
            "Metrics": [
                {"MetricName": "CPUUtilization"},
            ]
        }
        mock_cloudwatch_client = MagicMock()
        mock_cloudwatch_client.list_metrics.return_value = mock_list_metrics_return

        metrics_list = get_metrics(
            job_id=job_id,
            s3_bucket="some bucket",
            module="tabular",
            benchmark_name="some_benchmark",
            sub_folder="test_folder",
            cloudwatch_client=mock_cloudwatch_client,
        )
        self.assertEqual(metrics_list, metrics)

    @patch("tempfile.TemporaryDirectory")
    @patch("autogluon.bench.eval.hardware_metrics.hardware_metrics.save_results")
    @patch("autogluon.bench.eval.hardware_metrics.hardware_metrics.upload_to_s3")
    @patch("autogluon.bench.eval.hardware_metrics.hardware_metrics.get_metrics")
    @patch("boto3.client")
    def test_get_hardware_metrics(self, mock_boto3, mock_metrics, mock_upload_to_s3, mock_save_results, mock_temp_dir):
        mock_metrics.return_value = ["metrics"]
        get_hardware_metrics(config_file, "some bucket", "tabular", "some_benchmark")
        job_ids = get_job_ids(config)
        calls = [
            call(
                job_id=job_ids[0],
                s3_bucket="some bucket",
                module="tabular",
                benchmark_name="some_benchmark",
                sub_folder="some_benchmark_123456-abc-efg",
                cloudwatch_client=ANY,
            ),
            call(
                job_id=job_ids[1],
                s3_bucket="some bucket",
                module="tabular",
                benchmark_name="some_benchmark",
                sub_folder="some_benchmark_010101-xxx-zzz",
                cloudwatch_client=ANY,
            ),
        ]
        mock_metrics.assert_has_calls(calls, any_order=False)
        mock_save_results.assert_called_once()
        mock_upload_to_s3.assert_called_once()

    def test_invalid_config_file(self):
        with self.assertRaises(ValueError):
            get_hardware_metrics(None, "some bucket", "tabular", "some_benchmark")
        with self.assertRaises(FileNotFoundError):
            get_hardware_metrics("incorrect config path", "some bucket", "tabular", "some_benchmark")


if __name__ == "__main__":
    unittest.main()



import logging
import os

import pandas as pd
import yaml

from autogluon.bench.utils.dataset_utils import get_data_home_dir
from autogluon.common.loaders import load_zip


def path_expander(path, base_folder):
    path_l = path.split(";")
    return ";".join([os.path.abspath(os.path.join(base_folder, path)) for path in path_l])


logger = logging.getLogger(__name__)


class VisionDataLoader:
    def __init__(self, dataset_name: str, dataset_config_file: str, split: str = "train"):
        with open(dataset_config_file, "r") as f:
            config = yaml.safe_load(f)

        self.dataset_config = config[dataset_name]
        if split == "val":
            split = "validation"
        if split not in self.dataset_config["splits"]:
            logger.warning(f"Data split {split} not available.")
            self.data = None
            return

        self.name = dataset_name
        self.split = split
        self.feature_columns = self.dataset_config["feature_columns"]
        self.label_columns = self.dataset_config["label_columns"]

        url = self.dataset_config["url"].format(name=self.name)
        base_dir = get_data_home_dir()
        load_zip.unzip(url, unzip_dir=base_dir)
        self.dataset_dir = os.path.join(base_dir, self.name)

        annotation_filename = self.dataset_config["annotation"].format(name=self.name, split=self.split)
        image_path_pattern = self.dataset_config["image_path"]

        self.data = pd.read_csv(os.path.join(self.dataset_dir, annotation_filename))
        _columns_to_drop = self.data.columns.difference(self.feature_columns + self.label_columns)
        self.data.drop(columns=_columns_to_drop, inplace=True)
        image_base_path = image_path_pattern.format(name=self.name, split=self.split, value="")
        for col in self.feature_columns:
            self.data[col] = self.data[col].apply(
                lambda ele: path_expander(ele, base_folder=os.path.join(self.dataset_dir, image_base_path))
            )

    @property
    def problem_type(self):
        return self.dataset_config["problem_type"]

    @property
    def metric(self):
        return self.dataset_config["metric"]



import abc
import logging
import os

from autogluon.bench.utils.dataset_utils import get_data_home_dir, get_repo_url
from autogluon.common.loaders import load_zip

from .constants import _OBJECT_DETECTION

# Add dataset class names here
__all__ = ["TinyMotorbike", "Clipart"]

logger = logging.getLogger(__name__)


class BaseObjectDetectionDataset(abc.ABC):
    def __init__(self, split: str, dataset_name: str, data_info: dict):
        """
        Initializes the class.

        Args:
            split (str): Specifies the dataset split. It should be one of the following options: 'train', 'val', 'test'.
        """
        self._path = os.path.join(get_data_home_dir(), dataset_name)
        load_zip.unzip(data_info["data"]["url"], unzip_dir=self._path)
        self._base_folder = os.path.join(self._path, dataset_name)
        self._data_path = os.path.join(self._base_folder, "Annotations", f"{split}_cocoformat.json")
        if not os.path.exists(self._data_path):
            logger.warn(f"No annotation found at {self._data_path}")
            self._data_path = None

    @property
    @abc.abstractmethod
    def base_folder(self):
        pass

    @property
    @abc.abstractmethod
    def data(self):
        pass

    @property
    @abc.abstractmethod
    def metric(self):
        pass

    @property
    def problem_type(self):
        return _OBJECT_DETECTION


class TinyMotorbike(BaseObjectDetectionDataset):
    _SOURCE = ""
    _INFO = {
        "data": {
            "url": get_repo_url() + "object_detection_dataset/tiny_motorbike_coco.zip",
        },
    }
    _registry_name = "tiny_motorbike"

    def __init__(self, split="train"):
        self._split = f"{split}val" if split == "train" else split
        super().__init__(split=self._split, dataset_name=self._registry_name, data_info=self._INFO)

    @property
    def base_folder(self):
        return self._base_folder

    @property
    def data(self):
        return self._data_path

    @property
    def metric(self):
        return "map"


class Clipart(BaseObjectDetectionDataset):
    _SOURCE = "https://github.com/naoto0804/cross-domain-detection/tree/master/datasets"
    _INFO = {
        "data": {
            "url": get_repo_url() + "few_shot_object_detection/clipart.zip",
        },
    }
    _registry_name = "clipart"

    def __init__(self, split="train"):
        self._split = split
        super().__init__(split=self._split, dataset_name=self._registry_name, data_info=self._INFO)

    @property
    def base_folder(self):
        return self._base_folder

    @property
    def data(self):
        return self._data_path

    @property
    def metric(self):
        return "map"



import io
import itertools
import logging
import os
import re
import zipfile

import requests
import yaml
from boto3 import client

aws_batch = client("batch")
s3 = client("s3")

logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)

AMLB_DEPENDENT_MODULES = ["tabular", "timeseries"]


def submit_batch_job(env: list, job_name: str, job_queue: str, job_definition: str, array_size: int):
    """
    Submits a Batch job with the given environment variables, job name, job queue and job definition.

    Args:
        env (List[Dict[str, Any]]): List of dictionaries containing environment variables.
        job_name (str): Name of the job.
        job_queue (str): Name of the job queue.
        job_definition (str): Name of the job definition.
        array_size (int): Number of jobs to submit.

    Returns:
        str: Job ID.
    """
    container_overrides = {"environment": env}
    job_params = {
        "jobName": job_name,
        "jobQueue": job_queue,
        "jobDefinition": job_definition,
        "containerOverrides": container_overrides,
    }
    if array_size > 1:
        job_params["arrayProperties"] = {"size": array_size}

    response = aws_batch.submit_job(**job_params)

    logger.info("Job %s submitted to AWS Batch queue %s.", job_name, job_queue)
    logger.info(response)
    return response["jobId"]


def download_file_from_s3(s3_path: str, local_path: str) -> str:
    """Downloads a file from an S3 bucket.

    Args:
        s3_path (str): The S3 path of the file.
        local_path (str): The local path where the file will be downloaded.

    Returns:
        str: The local path of the downloaded file.
    """
    bucket = s3_path.strip("s3://").split("/")[0]
    s3_path = s3_path[len(f"s3://{bucket}/") :]

    local_file_path = os.path.join(local_path, s3_path.split("/")[-1])
    s3.download_file(bucket, s3_path, local_file_path)

    return local_file_path


def download_dir_from_s3(s3_path: str, local_path: str) -> str:
    """Downloads a directory from an S3 bucket.

    Args:
        s3_path (str): The S3 path of the directory.
        local_path (str): The local path where the directory will be downloaded.

    Returns:
        str: The local path of the downloaded directory.
    """
    bucket = s3_path.strip("s3://").split("/")[0]
    s3_path = s3_path[len(f"s3://{bucket}/") :]

    response = s3.list_objects(Bucket=bucket, Prefix=s3_path)

    for content in response.get("Contents", []):
        s3_obj_path = content["Key"]
        relative_path = os.path.relpath(s3_obj_path, s3_path)
        local_obj_path = os.path.join(local_path, relative_path)

        os.makedirs(os.path.dirname(local_obj_path), exist_ok=True)
        s3.download_file(bucket, s3_obj_path, local_obj_path)

    return local_path


def upload_config(config_list: list, bucket: str, benchmark_name: str):
    """
    Uploads a file to the given S3 bucket.

    Args:
        bucket (str): Name of the bucket.
        file (str): Local path of the file to upload.

    Returns:
        str: S3 path of the uploaded file.
    """
    s3_key = f"configs/{benchmark_name}/{benchmark_name}_job_configs.yaml"
    s3.put_object(Body=yaml.dump(config_list), Bucket=bucket, Key=s3_key)
    return f"s3://{bucket}/{s3_key}"


def download_automlbenchmark_resources():
    """
    Clones the stable version of the AutoML Benchmark repository from GitHub as a zip file,
    and extracts all files under 'resources' directory into a local path.

    Returns:
        str: The local path of the cloned repository.
    """

    amlb_zip = "https://github.com/openml/automlbenchmark/archive/refs/tags/stable.zip"
    resources_path = "automlbenchmark-stable/resources"
    automlbenchmark_repo_path = "/tmp"
    response = requests.get(amlb_zip)

    with zipfile.ZipFile(io.BytesIO(response.content)) as z:
        for filename in z.namelist():
            if filename.startswith(resources_path):
                destination_path = os.path.join(automlbenchmark_repo_path, filename)
                if filename.endswith("/"):
                    # This is a directory, need to create it
                    os.makedirs(destination_path, exist_ok=True)
                else:
                    # Extract the file to the destination path
                    with z.open(filename) as zf, open(destination_path, "wb") as f:
                        f.write(zf.read())
    return str(os.path.join(automlbenchmark_repo_path, "automlbenchmark-stable"))


def load_benchmark_from_yaml(filepath):
    with open(filepath, "r") as file:
        benchmark_yaml = yaml.safe_load(file)
    return [task["name"] for task in benchmark_yaml]


def find_benchmark_yaml(benchmark, search_dirs):
    for dir in search_dirs:
        filepath = os.path.join(dir, benchmark + ".yaml")
        if os.path.isfile(filepath):
            return filepath
    return None


def get_run_folds(file: str, default_max_folds: int = 10):
    configs = {}
    with open(file, "r") as f:
        amlb_benchmark_configs = yaml.safe_load(f)
        for item in amlb_benchmark_configs:
            folds = min(item.get("folds", default_max_folds), default_max_folds)
            configs[item["name"]] = [i for i in range(folds)]
    return configs


def get_max_fold(amlb_constraint_search_files: list, constraint: str):
    default_folds = 10
    for file in amlb_constraint_search_files:
        with open(file, "r") as f:
            constraints = yaml.safe_load(f)
            if constraint in constraints.keys():
                return constraints[constraint].get("folds", default_folds)
    return default_folds


def process_benchmark_runs(module_configs: dict, amlb_benchmark_search_dirs: list, default_max_folds: int = 10):
    """
    Updates module_configs["fold_to_run"] with tasks and folds defined in config files

    Only amlb_benchmark was required,
    if amlb_task does not have keys, we assume all tasks in the amlb_benchmark will be run;
    similarly, if folds_to_run does not have any key, we assume all folds in amlb_task will be run.
    If the aforementioned have keys, we only populate folds_to_run for the existing keys.
    """
    module_configs.setdefault("amlb_task", {})
    module_configs.setdefault("fold_to_run", {})

    amlb_task_folds = {}
    for benchmark in module_configs["amlb_benchmark"]:
        filepath = find_benchmark_yaml(benchmark, amlb_benchmark_search_dirs)
        if filepath is not None:
            amlb_task_folds[benchmark] = get_run_folds(file=filepath, default_max_folds=default_max_folds)
        else:
            raise FileNotFoundError(f"Benchmark config for {benchmark} is not found.")

        if module_configs["amlb_task"].get(benchmark) is None:
            module_configs["amlb_task"][benchmark] = list(amlb_task_folds[benchmark].keys())

        module_configs["fold_to_run"].setdefault(benchmark, {})
        for task in module_configs["amlb_task"][benchmark]:
            if module_configs["fold_to_run"][benchmark].get(task):
                folds = module_configs["fold_to_run"][benchmark][task]
            else:
                folds = amlb_task_folds[benchmark][task]
            module_configs["fold_to_run"][benchmark][task] = [f for f in folds if f < default_max_folds]
            if not module_configs["fold_to_run"][benchmark][task]:
                del module_configs["fold_to_run"][benchmark][task]


def get_cloudwatch_logs_url(region: str, job_id: str, log_group_name: str = "aws/batch/job"):
    base_url = f"https://console.aws.amazon.com/cloudwatch/home?region={region}"
    job_response = aws_batch.describe_job(jobs=[job_id])
    log_stream_name = job_response["jobs"][0]["attempts"][0]["container"]["logStreamName"]
    return f"{base_url}#logsV2:log-groups/log-group/{log_group_name.replace('/', '%2F')}/log-events/{log_stream_name.replace('/', '%2F')}"


def generate_config_combinations(config, metrics_bucket, batch_job_queue, batch_job_definition):
    job_configs = []
    if config["module"] in AMLB_DEPENDENT_MODULES:
        job_configs = generate_amlb_module_config_combinations(config)
    elif config["module"] == "multimodal":
        job_configs = generate_multimodal_config_combinations(config)
    else:
        raise ValueError("Invalid module. Choose either 'tabular', 'timeseries', or 'multimodal'.")

    if len(job_configs) == 0:
        return {parent_job_id: "No job submitted"}

    benchmark_name = config["benchmark_name"]
    config_s3_path = upload_config(config_list=job_configs, bucket=metrics_bucket, benchmark_name=benchmark_name)
    env = [{"name": "config_file", "value": config_s3_path}]
    job_type = "array" if len(job_configs) > 1 else "single"
    constraint = config.get("amlb_constraint") or config.get("constraint")
    job_name = f"{benchmark_name}-{config['module']}-{config['framework']}-{constraint}-{job_type}-job"
    job_name = re.sub(r"(?![-_])\W", "-", job_name)[
        :128
    ]  # AWS Bath Job name can only contain letters, numbers, "-" and "_"

    parent_job_id = submit_batch_job(
        env=env,
        job_name=job_name,
        job_queue=batch_job_queue,
        job_definition=batch_job_definition,
        array_size=len(job_configs),
    )
    return {parent_job_id: config_s3_path}


def generate_multimodal_config_combinations(config):
    common_keys = []
    specific_keys = []
    for key in config.keys():
        if isinstance(config[key], list):
            specific_keys.append(key)
        else:
            common_keys.append(key)

    specific_value_combinations = list(
        itertools.product(*(config[key] for key in specific_keys if key in config.keys()))
    ) or [None]

    all_configs = []
    for combo in specific_value_combinations:
        new_config = {key: config[key] for key in common_keys}
        if combo is not None:
            new_config.update(dict(zip(specific_keys, combo)))
        all_configs.append(new_config)

    return all_configs


def generate_amlb_module_config_combinations(config):
    specific_keys = ["git_uri#branch", "framework", "amlb_constraint", "amlb_user_dir"]
    exclude_keys = ["amlb_benchmark", "amlb_task", "fold_to_run"]
    common_keys = []
    specific_keys = []
    for key in config.keys():
        if key in exclude_keys:
            continue

        if isinstance(config[key], list):
            specific_keys.append(key)
        else:
            common_keys.append(key)

    specific_value_combinations = list(
        itertools.product(*(config[key] for key in specific_keys if key in config.keys()))
    ) or [None]

    # Iterate through the combinations and the amlb benchmark task keys
    # Generates a config for each combination of specific key and keys in `fold_to_run`
    all_configs = []
    for combo in specific_value_combinations:
        for benchmark, tasks in config["fold_to_run"].items():
            for task, fold_numbers in tasks.items():
                for fold_num in fold_numbers:
                    new_config = {key: config[key] for key in common_keys}
                    if combo is not None:
                        new_config.update(dict(zip(specific_keys, combo)))
                    new_config["amlb_benchmark"] = benchmark
                    new_config["amlb_task"] = task
                    new_config["fold_to_run"] = fold_num
                    all_configs.append(new_config)

    return all_configs


def handler(event, context):
    """
    Execution entrypoint for AWS Lambda.
    Triggers batch jobs with hyperparameter combinations.
    ENV variables are set by the AWS CDK infra code.
    """
    if "config_file" not in event or not event["config_file"].startswith("s3"):
        raise KeyError("S3 path of config file is required.")

    config_file_path = download_file_from_s3(s3_path=event["config_file"], local_path="/tmp")
    with open(config_file_path, "r") as f:
        configs = yaml.safe_load(f)

    metrics_bucket = configs["cdk_context"]["METRICS_BUCKET"]

    batch_job_queue = os.environ.get("BATCH_JOB_QUEUE")
    batch_job_definition = os.environ.get("BATCH_JOB_DEFINITION")

    configs["METRICS_BUCKET"] = metrics_bucket
    configs["mode"] = "local"

    if configs["module"] in AMLB_DEPENDENT_MODULES:
        # download the almb repo resources/ to process the default resources
        amlb_repo_path = download_automlbenchmark_resources()
        amlb_benchmark_search_dirs = []
        amlb_constraint_search_files = []
        amlb_user_dir = configs.get("amlb_user_dir")
        if amlb_user_dir is not None:
            if amlb_user_dir.startswith("s3://"):
                amlb_user_dir_local = download_dir_from_s3(
                    s3_path=amlb_user_dir, local_path=f"/tmp/amlb_custom_configs"
                )
            else:
                amlb_user_dir_local = amlb_user_dir

            # check if default amlb resources are required
            user_config_file = os.path.join(amlb_user_dir_local, "config.yaml")
            with open(user_config_file, "r") as f:
                # check the user_dir config.yaml and append search directories accordingly
                user_configs = yaml.safe_load(f)
                if user_configs.get("benchmarks"):
                    if user_configs["benchmarks"].get("definition_dir"):
                        for definition_dir in user_configs["benchmarks"]["definition_dir"]:
                            if "{user}" in definition_dir:
                                ext = definition_dir.split("{user}")[-1].lstrip("/")
                                amlb_benchmark_search_dirs.append(os.path.join(amlb_user_dir_local, ext))
                            elif "{root}" in definition_dir:
                                ext = definition_dir.split("{root}")[-1].lstrip("/")
                                amlb_benchmark_search_dirs.append(os.path.join(amlb_repo_path, ext))
                    else:
                        amlb_benchmark_search_dirs.append(os.path.join(amlb_repo_path, "resources/benchmarks"))

                    if user_configs["benchmarks"].get("constraints_file"):
                        for constraints_file in user_configs["benchmarks"]["constraints_file"]:
                            if "{user}" in constraints_file:
                                ext = constraints_file.split("{user}")[-1].lstrip("/")
                                amlb_constraint_search_files.append(os.path.join(amlb_user_dir_local, ext))
                            elif "{root}" in constraints_file:
                                ext = constraints_file.split("{root}")[-1].lstrip("/")
                                amlb_constraint_search_files.append(os.path.join(amlb_repo_path, ext))
                    else:
                        amlb_constraint_search_files.append(os.path.join(amlb_repo_path, "constraints.yaml"))
        else:
            # if no amlb_user_dir is specified, default resources/ from amlb stable version is used
            amlb_benchmark_search_dirs.append(os.path.join(amlb_repo_path, "resources/benchmarks"))
            amlb_constraint_search_files.append(os.path.join(amlb_repo_path, "resources/constraints.yaml"))

        if configs.get("amlb_constraint") is None:
            configs["amlb_constraint"] = "test"
        fold_constraint = get_max_fold(
            amlb_constraint_search_files=amlb_constraint_search_files, constraint=configs["amlb_constraint"]
        )
        process_benchmark_runs(
            module_configs=configs,
            amlb_benchmark_search_dirs=amlb_benchmark_search_dirs,
            default_max_folds=fold_constraint,
        )

    response = generate_config_combinations(
        config=configs,
        metrics_bucket=metrics_bucket,
        batch_job_queue=batch_job_queue,
        batch_job_definition=batch_job_definition,
    )

    return response



import json
import logging
import os
import subprocess
import sys
from typing import Optional

from autogluon.bench import __version__ as agbench_version
from autogluon.bench.frameworks.benchmark import Benchmark

logger = logging.getLogger(__name__)


class MultiModalBenchmark(Benchmark):
    """
    A benchmark class for AutoGluon MultiModal.

    Attributes:
        benchmark_name (str): The name of the benchmark.
        root_dir (str): The root directory for the benchmark.
        module (str): The name of the module being benchmarked (multimodal).

    Methods:
        setup(): Sets up the virtual environment for running the benchmark.
        run(): Runs the benchmark on a given dataset.
    """

    def setup(
        self,
        git_uri: str = "https://github.com/autogluon/autogluon.git",
        git_branch: str = "master",
    ):
        """
        Sets up the virtual environment for running the benchmark.

        Args:
            git_uri (str): The URI of the Git repository to clone (default: "https://github.com/autogluon/autogluon.git").
            git_branch (str): The branch of the Git repository to clone (default: "master").

        Returns:
            None
        """
        setup_script_path = os.path.join(os.path.abspath(os.path.dirname(__file__)), "setup.sh")
        command = [setup_script_path, git_uri, git_branch, self.benchmark_dir, agbench_version]
        result = subprocess.run(command)
        if result.returncode != 0:
            sys.exit(1)
        else:
            logger.info("Successfully set up the environment under %s/.venv.", self.benchmark_dir)

    def run(
        self,
        dataset_name: str,
        framework: str,
        constraint: Optional[str] = None,
        params: Optional[dict] = None,
        custom_dataloader: Optional[dict] = None,
        custom_metrics: Optional[dict] = None,
        time_limit: Optional[int] = None,
    ):
        """
        Runs the benchmark on a given dataset.

        Args:
            dataset_name (str): Dataset name, can be registered with multimodal_dataset_registry or a custom dataset.

                                To get a list of datasets:
                                from autogluon.bench.datasets.dataset_registry import multimodal_dataset_registry
                                multimodal_dataset_registry.list_keys()
            framework (str): The name of the framework to use for the benchmark.
            constraint (str): The resource constraint used by benchmarking during AWS mode.
            params (str): The multimodal params.
            custom_dataloader (Optional[dict], None): A dictionary containing information about a custom dataloader to use. Defaults to None.
                                To define a custom dataloader in the config file:

                                custom_dataloader:
                                    dataloader_file: path_to/dataloader.py   # relative path to WORKDIR
                                    class_name: DataLoaderClass
                                    dataset_config_file: path_to/dataset_config.yaml
                                    **kwargs (of DataLoaderClass)
            custom_metrics (Optional[dict], None): A dictionary containing information about a custom metrics to use. Defaults to None.
                                To define a custom metrics in the config file:

                                custom_metrics:
                                    metrics_path: path_to/metrics.py   # relative path to WORKDIR
                                    function_name: custom_metrics_function
                                    **kwargs (of )
            time_limit (Optional[int], None): Time limit for predictor.fit()

        Returns:
            None
        """
        if os.environ.get("RUNNING_IN_DOCKER", "false") == "true":
            venv_base_dir = os.environ["VENV_BASE_DIR"]
        else:
            venv_base_dir = self.benchmark_dir
        PY_EXC_PATH = os.path.join(venv_base_dir, ".venv/bin/python")

        # Workaround for torch import issue with torch==2.5.x and cuda=12.x
        # https://github.com/pytorch/pytorch/issues/111469#issuecomment-1773937884
        site_packages_path = (
            subprocess.check_output(
                [PY_EXC_PATH, "-c", "import site; print(site.getsitepackages()[0] + '/nvidia/nvjitlink/lib')"]
            )
            .decode("utf-8")
            .strip()
        )
        env = os.environ.copy()
        env["LD_LIBRARY_PATH"] = f"{site_packages_path}:{env.get('LD_LIBRARY_PATH', '')}"

        exec_path = os.path.join(os.path.abspath(os.path.dirname(__file__)), "exec.py")
        logger.info(f"Executing {exec_path} under {PY_EXC_PATH}")
        command = [
            PY_EXC_PATH,
            exec_path,
            "--dataset_name",
            dataset_name,
            "--framework",
            framework,
            "--benchmark_dir",
            self.benchmark_dir,
            "--metrics_dir",
            self.metrics_dir,
        ]
        if constraint is not None:
            command += ["--constraint", constraint]
        if params is not None:
            command += ["--params", json.dumps(params)]
        if custom_dataloader is not None:
            command += ["--custom_dataloader", json.dumps(custom_dataloader)]
        if custom_metrics is not None:
            command += ["--custom_metrics", json.dumps(custom_metrics)]
        if time_limit is not None:
            command += ["--time_limit", str(time_limit)]
        result = subprocess.run(command, env=env)
        if result.returncode != 0:
            sys.exit(1)
        else:
            logger.info(f"Benchmark {self.benchmark_name} on dataset {dataset_name} is complete.")



import pytest
from aws_cdk import Environment

env = Environment(account="123456789012", region="us-west-1")

default_configs = {
    "CDK_DEPLOY_ACCOUNT": "dummy",
    "CDK_DEPLOY_REGION": "us-west-2",
    "PREFIX": "test-prefix",
    "METRICS_BUCKET": "test-metrics-bucket",
    "DATA_BUCKET": "test-data-bucket",
    "INSTANCE": "g4dn.2xlarge",
    "MAX_MACHINE_NUM": 2,
    "RESERVED_MEMORY_SIZE": 10000,
    "TIME_LIMIT": 3600,
    "BLOCK_DEVICE_VOLUME": 10,
    "VPC_NAME": "test-vpc",
}

context_values = {
    "STACK_NAME_PREFIX": "test",
    "STACK_NAME_TAG": "benchmark",
    "STATIC_RESOURCE_STACK_NAME": "test-static-resource-stack",
    "BATCH_STACK_NAME": "test-batch-stack",
    "METRICS_BUCKET": "test-metrics-bucket",
    "DATA_BUCKET": "test-data-bucket",
    "INSTANCE_TYPES": ["g4dn.2xlarge"],
    "COMPUTE_ENV_MAXV_CPUS": 160,
    "CONTAINER_GPU": 1,
    "CONTAINER_VCPU": 8,
    "CONTAINER_MEMORY": 10000,
    "TIME_LIMIT": 3600,
    "BLOCK_DEVICE_VOLUME": 100,
    "LAMBDA_FUNCTION_NAME": "test-batch-job-function",
}


@pytest.fixture
def default_context_values():
    return context_values



import os
import unittest
from unittest.mock import MagicMock

import pytest

from autogluon.bench.frameworks.benchmark import Benchmark


class TempBenchmark(Benchmark):
    def run(self):
        pass


@pytest.fixture(scope="function")
def benchmark():
    return TempBenchmark("test_benchmark", "test_dir")


def test_attributes(benchmark):
    assert benchmark.benchmark_name.startswith("test_benchmark")
    assert benchmark.benchmark_dir.startswith("test_dir")
    assert benchmark.metrics_dir.startswith("test_dir")


def test_upload_metrics(benchmark, temp_dir):
    benchmark.metrics_dir = temp_dir

    temp_file = os.path.join(temp_dir, "file.txt")
    with open(temp_file, "w") as f:
        f.write("test")

    with unittest.mock.patch("boto3.client") as mock_client:
        mock_upload_file = MagicMock()
        mock_client.return_value = MagicMock(upload_file=mock_upload_file)

        s3_bucket = "my-bucket"
        s3_dir = "my-s3-dir"
        benchmark.upload_metrics(s3_bucket, s3_dir)

        assert mock_upload_file.call_count == 1

        s3_path = mock_upload_file.call_args[0][2]
        assert s3_path.startswith(f"{s3_dir}/file.txt")



import os
import warnings
from pathlib import Path

import pandas as pd

_PATH_TO_DATA = str(Path(__file__).parent.parent.parent.parent.parent.parent.parent / "data" / "metadata")


def load_task_metadata(path="task_metadata.csv", path_relative_to_data=True, compute_aux_columns=True):
    if path_relative_to_data:
        path = _PATH_TO_DATA + os.sep + path
    task_metadata = pd.read_csv(path)
    if not compute_aux_columns:
        return task_metadata
    task_metadata["ClassRatio"] = task_metadata["MinorityClassSize"] / task_metadata["NumberOfInstances"]

    with warnings.catch_warnings():
        warnings.filterwarnings("ignore")
        task_metadata["problem_type"] = ""
        task_metadata["problem_type"][task_metadata["NumberOfClasses"] == 2] = "binary"
        task_metadata["problem_type"][task_metadata["NumberOfClasses"] > 2] = "multiclass"
        task_metadata["problem_type"][task_metadata["NumberOfClasses"] == 0] = "regression"
    task_metadata["NumberOfSymbolicFeatures"] = [
        num_sym if num_class == 0 else num_sym - 1
        for num_sym, num_class in zip(task_metadata["NumberOfSymbolicFeatures"], task_metadata["NumberOfClasses"])
    ]
    task_metadata["NumberOfNumericFeatures"] = [
        num_num if num_class != 0 else num_num - 1
        for num_num, num_class in zip(task_metadata["NumberOfNumericFeatures"], task_metadata["NumberOfClasses"])
    ]
    task_metadata["NumberOfFeatures"] -= 1
    return task_metadata



import typer

from autogluon.bench.eval.aggregate.results import aggregate_results

app = typer.Typer()


@app.command()
def aggregate_amlb_results(
    s3_bucket: str = typer.Argument(help="Name of the S3 bucket to which the aggregated results will be outputted."),
    module: str = typer.Argument(help="Can be one of ['tabular', 'timeseries', 'multimodal']."),
    benchmark_name: str = typer.Argument(
        help="Folder name of benchmark run in which all objects with path 'scores/results.csv' get aggregated."
    ),
    constraint: str = typer.Option(
        None,
        help="Name of constraint used in benchmark, refer to https://github.com/openml/automlbenchmark/blob/master/resources/constraints.yaml. Not applicable when `module==multimodal`",
    ),
    include_infer_speed: bool = typer.Option(False, help="Include inference speed in aggregation."),
    mode: str = typer.Option("ray", help='Aggregation mode: "seq" or "ray".'),
):
    """
    Finds "scores/results.csv" under s3://<s3_bucket>/<module>/<benchmark_name> recursively with the constraint if provided,
    and append all results into one file at s3://<s3_bucket>/aggregated/<module>/<benchmark_name>/results_automlbenchmark_<constraint>_<benchmark_name>.csv

    Example:
        agbench aggregate-amlb-results autogluon-benchmark-metrics tabular ag_tabular_20230629T140546 --constraint test
    """

    aggregate_results(
        s3_bucket=s3_bucket,
        s3_prefix=f"{module}/",
        version_name=benchmark_name,
        constraint=constraint,
        include_infer_speed=include_infer_speed,
        mode=mode,
    )


if __name__ == "__main__":
    app()



#!/usr/bin/env python3

import importlib.resources
import logging
import os

import aws_cdk as core
import boto3
from aws_cdk import aws_batch as batch
from aws_cdk import aws_ec2 as ec2
from aws_cdk import aws_ecr_assets as ecr_assets
from aws_cdk import aws_iam as iam
from aws_cdk import aws_s3 as s3
from constructs import Construct

from autogluon.bench.cloud.aws.batch_stack.constructs.batch_lambda_function import BatchLambdaFunction

"""
Sample CDK code for creating the required infrastructure for running a AWS Batch job.
AWS Batch as the compute enviroment in which a docker image runs the benchmarking script.
"""


def find_project_root_or_fallback(start_dir: str, root_identifier: str = "pyproject.toml"):
    """Find the project root directory by searching for a specific identifier file.
    If not found, fall back to the starting directory.
    """
    current_dir = start_dir

    while current_dir != os.path.dirname(current_dir):
        if os.path.exists(os.path.join(current_dir, root_identifier)):
            return str(current_dir)
        current_dir = os.path.dirname(current_dir)

    return start_dir


# when pip installed, project_root is at $site_package_path/src/autogluon/bench agbench_base_dir should be ./
# when installed from source, project_root is at ./, agbench_base_dir should be ./src/autogluon/bench
with importlib.resources.path("autogluon.bench", "Dockerfile") as file_path:
    docker_base_dir = os.path.dirname(file_path)
    project_root = find_project_root_or_fallback(docker_base_dir)
    docker_path = os.path.relpath(file_path, project_root)
    agbench_base_dir = os.path.dirname(docker_path)
    if agbench_base_dir == "":
        agbench_base_dir = "."

with importlib.resources.path("autogluon.bench.cloud.aws.batch_stack.lambdas", "lambda_function.py") as file_path:
    lambda_script_dir = os.path.dirname(file_path)

logger = logging.getLogger(__name__)


class StaticResourceStack(core.Stack):
    """
    Defines a stack for creating and importing static resources, such as S3 buckets and VPCs.
    """

    def import_or_create_bucket(self, resource, bucket_name):
        """
        Imports an S3 bucket if it already exists or creates a new one if it doesn't exist.

        Args:
            resource: A boto3 S3 resource object.
            bucket_name: The name of the S3 bucket.

        Returns:
            An S3 bucket object.
        """
        bucket = resource.Bucket(bucket_name)
        if bucket.creation_date:
            logger.warning("The bucket %s already exists, importing to the stack...", bucket_name)
            bucket = s3.Bucket.from_bucket_name(self, bucket_name, bucket_name=bucket_name)
        else:
            logger.warning("The bucket %s does not exist, creating a new bucket...", bucket_name)
            bucket = s3.Bucket(
                self,
                bucket_name,
                bucket_name=bucket_name,
                removal_policy=core.RemovalPolicy.RETAIN,
            )
        return bucket

    def create_s3_resources(self):
        """
        Creates S3 bucket resources.
        """
        region = os.environ["CDK_DEPLOY_REGION"]
        s3_resource = boto3.resource(service_name="s3", region_name=region)
        self.metrics_bucket = self.import_or_create_bucket(resource=s3_resource, bucket_name=self.metrics_bucket_name)
        if self.data_bucket_name:
            self.data_bucket = s3.Bucket.from_bucket_name(
                self, self.data_bucket_name, bucket_name=self.data_bucket_name
            )
        else:
            self.data_bucket = None

    def __init__(self, scope: Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)
        self.metrics_bucket_name = self.node.try_get_context("METRICS_BUCKET")
        self.data_bucket_name = self.node.try_get_context("DATA_BUCKET")
        self.vpc_name = self.node.try_get_context("VPC_NAME")
        self.prefix = self.node.try_get_context("STACK_NAME_PREFIX")

        self.create_s3_resources()
        self.vpc = ec2.Vpc.from_lookup(self, f"{self.prefix}-vpc", vpc_name=self.vpc_name) if self.vpc_name else None


class BatchJobStack(core.Stack):
    def __init__(self, scope: Construct, id: str, static_stack: StaticResourceStack, **kwargs) -> None:
        """
        Defines a stack with the following:
        - A new Compute Environment with
            - Batch Instance Role (read access to S3)
            - Launch Template
            - Security Group
            - VPC
            - Job Definition (with customized container)
            - Job Queue
        - An existing or new S3 bucket
        - A new Lambda function to run training.

        Args:
            scope (constructs.Construct): The scope of the stack.
            id (str): The ID of the stack.
            static_stack (StaticResourceStack): A StaticResourceStack object.
            **kwargs: Keyword arguments.
        """
        super().__init__(scope, id, **kwargs)
        prefix = self.node.try_get_context("STACK_NAME_PREFIX")
        tag = self.node.try_get_context("STACK_NAME_TAG")
        instance_types = self.node.try_get_context("INSTANCE_TYPES")
        compute_env_maxv_cpus = int(self.node.try_get_context("COMPUTE_ENV_MAXV_CPUS"))
        container_gpu = self.node.try_get_context("CONTAINER_GPU")
        container_vcpu = self.node.try_get_context("CONTAINER_VCPU")
        container_memory = self.node.try_get_context("CONTAINER_MEMORY")
        time_limit = self.node.try_get_context("TIME_LIMIT")
        block_device_volume = self.node.try_get_context("BLOCK_DEVICE_VOLUME")
        lambda_function_name = self.node.try_get_context("LAMBDA_FUNCTION_NAME") + "-" + prefix

        vpc = static_stack.vpc

        if vpc is None:
            vpc = ec2.Vpc(
                self,
                f"{prefix}-vpc",
                max_azs=2,  # This number can be increased for high availability
                nat_gateways=1,
                subnet_configuration=[
                    ec2.SubnetConfiguration(
                        name=f"{prefix}-PublicSubnet",
                        subnet_type=ec2.SubnetType.PUBLIC,
                    ),
                    ec2.SubnetConfiguration(
                        name=f"{prefix}-PrivateSubnet",
                        subnet_type=ec2.SubnetType.PRIVATE_WITH_EGRESS,
                    ),
                ],
            )
        sg = ec2.SecurityGroup(
            self,
            f"{prefix}-security-group",
            vpc=vpc,
        )

        # Add inbound rule for ssh access
        # sg.add_ingress_rule(
        #     peer=ec2.Peer.any_ipv4(),
        #     connection=ec2.Port.tcp(22),
        #     description="Allow SSH access from Internet"
        # )

        # Currently CDK can only push to the default repo aws-cdk/assets
        # https://github.com/aws/aws-cdk/issues/12597
        # TODO: use https://github.com/cdklabs/cdk-docker-image-deployment

        logger.info(f"Building Dockerfile at {docker_path} with context at {project_root}")
        image_name = f"{prefix}-ecr-docker-image-asset"
        docker_image_asset = ecr_assets.DockerImageAsset(
            self,
            image_name,
            directory=project_root,
            file=docker_path,
            follow_symlinks=core.SymlinkFollowMode.ALWAYS,
            build_args={
                "AG_BENCH_BASE_IMAGE": os.environ["AG_BENCH_BASE_IMAGE"],
                "AG_BENCH_BASE_DIR": agbench_base_dir,
                "AG_BENCH_VERSION": os.getenv("AG_BENCH_VERSION", "latest"),
                "CDK_DEPLOY_REGION": os.environ["CDK_DEPLOY_REGION"],
                "FRAMEWORK_PATH": os.environ["FRAMEWORK_PATH"],
                "GIT_URI": os.environ["GIT_URI"],
                "GIT_BRANCH": os.environ["GIT_BRANCH"],
                "AMLB_FRAMEWORK": os.getenv("AMLB_FRAMEWORK", ""),
                "AMLB_USER_DIR": os.getenv("AMLB_USER_DIR", ""),
            },
        )

        resource_requirements = [
            batch.CfnJobDefinition.ResourceRequirementProperty(type="VCPU", value=str(container_vcpu)),
            batch.CfnJobDefinition.ResourceRequirementProperty(type="MEMORY", value=str(container_memory)),
        ]

        if container_gpu > 0:
            resource_requirements.append(
                batch.CfnJobDefinition.ResourceRequirementProperty(type="GPU", value=str(container_gpu))
            )
        container_properties = batch.CfnJobDefinition.ContainerPropertiesProperty(
            image=docker_image_asset.image_uri,
            resource_requirements=resource_requirements,
            linux_parameters=batch.CfnJobDefinition.LinuxParametersProperty(shared_memory_size=container_memory),
        )

        job_definition_name = f"{prefix}-job-definition"
        job_definition = batch.CfnJobDefinition(
            self,
            job_definition_name,
            type="container",
            container_properties=container_properties,
            job_definition_name=job_definition_name,
            retry_strategy=batch.CfnJobDefinition.RetryStrategyProperty(attempts=3),
            timeout=batch.CfnJobDefinition.TimeoutProperty(attempt_duration_seconds=time_limit),
        )

        batch_launch_template_name = f"{prefix}-launch-template"
        launch_template = ec2.CfnLaunchTemplate(
            self,
            batch_launch_template_name,
            launch_template_name=batch_launch_template_name,
            launch_template_data={
                "blockDeviceMappings": [
                    {
                        "deviceName": "/dev/xvda",
                        "ebs": {
                            "volumeSize": block_device_volume,  # Ensure block_device_volume is defined
                            "volumeType": "gp3",
                            "deleteOnTermination": True,
                        },
                    }
                ],
                "metadataOptions": {"httpTokens": "required", "httpEndpoint": "enabled"},
            },
        )

        cloudwatch_policy = iam.Policy(
            self,
            f"{prefix}-cloudwatch-policy",
            policy_name=f"{prefix}-cloudwatch-policy",
            statements=[
                iam.PolicyStatement(
                    actions=["cloudwatch:PutMetricData"],
                    effect=iam.Effect.ALLOW,
                    resources=["*"],
                )
            ],
        )

        batch_service_role = iam.Role(
            self,
            f"{prefix}-batch-service-role",
            assumed_by=iam.ServicePrincipal("batch.amazonaws.com"),
            managed_policies=[iam.ManagedPolicy.from_aws_managed_policy_name("service-role/AWSBatchServiceRole")],
        )

        batch_instance_role = iam.Role(
            self,
            f"{prefix}-instance-role",
            assumed_by=iam.CompositePrincipal(
                iam.ServicePrincipal("ec2.amazonaws.com"),
                iam.ServicePrincipal("ecs.amazonaws.com"),
                iam.ServicePrincipal("ecs-tasks.amazonaws.com"),
            ),
            managed_policies=[
                iam.ManagedPolicy.from_aws_managed_policy_name("service-role/AmazonEC2ContainerServiceforEC2Role"),
            ],
        )
        batch_instance_role.attach_inline_policy(cloudwatch_policy)

        metrics_bucket = static_stack.metrics_bucket
        data_bucket = static_stack.data_bucket
        if data_bucket is not None:
            data_bucket.grant_read(batch_instance_role)
        metrics_bucket.grant_read_write(batch_instance_role)

        batch_instance_profile = iam.CfnInstanceProfile(
            self, f"{prefix}-instance-profile", roles=[batch_instance_role.role_name]
        )

        compute_environment = batch.CfnComputeEnvironment(
            self,
            f"{prefix}-compute-environment",
            type="MANAGED",
            service_role=batch_service_role.role_arn,
            compute_resources=batch.CfnComputeEnvironment.ComputeResourcesProperty(
                type="EC2",
                maxv_cpus=compute_env_maxv_cpus,
                minv_cpus=0,
                subnets=[subnet.subnet_id for subnet in vpc.private_subnets],
                # subnets=[subnet.subnet_id for subnet in vpc.public_subnets],  # use public subnet for ssh
                # ec2_key_pair=f"{prefix}-perm-key", # set this if you need ssh into instance
                allocation_strategy="BEST_FIT_PROGRESSIVE",
                instance_role=batch_instance_profile.attr_arn,
                instance_types=instance_types,
                security_group_ids=[sg.security_group_id],
                launch_template=batch.CfnComputeEnvironment.LaunchTemplateSpecificationProperty(
                    launch_template_name=batch_launch_template_name,
                ),
            ),
        )

        job_queue = batch.CfnJobQueue(
            self,
            f"{prefix}-job-queue",
            priority=1,
            compute_environment_order=[
                batch.CfnJobQueue.ComputeEnvironmentOrderProperty(compute_environment=compute_environment.ref, order=1)
            ],
            state="ENABLED",
        )

        lambda_function = BatchLambdaFunction(
            self,
            lambda_function_name,
            prefix=prefix,
            tag=tag,
            function_name=lambda_function_name,
            code_path=lambda_script_dir,
            environment={
                "BATCH_JOB_QUEUE": job_queue.ref,
                "BATCH_JOB_DEFINITION": job_definition.ref,
                "METRICS_BUCKET": metrics_bucket.bucket_name,
                "STACK_NAME_PREFIX": prefix,
            },
        )
        metrics_bucket.grant_read_write(lambda_function._lambda_function)

        # Output the ARN for manually updating tagging
        core.CfnOutput(
            self,
            "ComputeEnvironmentARN",
            value=compute_environment.ref,
        )
        core.CfnOutput(self, "JobQueueARN", value=job_queue.ref)
        core.CfnOutput(self, "JobDefinitionARN", value=job_definition.ref)
        core.CfnOutput(
            self,
            "EcrRepositoryName",
            value=docker_image_asset.repository.repository_name,
        )
        core.CfnOutput(self, "ImageUri", value=docker_image_asset.image_uri)



from __future__ import annotations

import os
from typing import Dict, List, Optional

import pandas as pd
import typer

from autogluon.bench.eval.evaluation import evaluate_results
from autogluon.bench.eval.evaluation.benchmark_evaluator import BenchmarkEvaluator
from autogluon.bench.eval.evaluation.constants import TIME_INFER_S
from autogluon.bench.eval.evaluation.evaluate_utils import (
    compute_stderr_z_stat,
    compute_stderr_z_stat_bulk,
    compute_win_rate_per_dataset,
    convert_to_dataset_fold,
    graph_vs,
)

app = typer.Typer()


@app.command()
def evaluate_amlb_results(
    results_dir: str = typer.Option("data/results/", help="Root directory of raw and prepared results"),
    results_dir_input: str = typer.Option(
        None,
        help="Directory of the results file '<file_prefix><constraint_str><benchmark_name_str>.csv' getting cleaned. Can be an S3 URI. If not provided, it defaults to '<results_dir>input/prepared/openml/'",
    ),
    results_dir_output: str = typer.Option(
        None,
        help="Output directory of evaluation files. If not provided, it defaults to '<results_dir>output/openml/<output_suffix>/'",
    ),
    paths: Optional[List[str]] = typer.Option(
        None,
        help="List of file paths under '<results_dir>input/prepared/openml/' or <results_dir_input> to load the input data from. Can also include files located in s3 assuming you have the proper read permissions. E.g. 'path1,path2,...",
    ),
    frameworks_run: Optional[List[str]] = typer.Option(
        None,
        help="List of framework to compare. These frameworks must be present in the 'framework' column of the loaded input files listed in the `paths` arg. E.g. 'framework1,framework2,...",
    ),
    output_suffix: str = typer.Option(
        "ag_eval",
        help="Output suffix of the path to save the output files, e.g. '<results_dir>output/openml/<output_suffix>/'.",
    ),
    framework_nan_fill: str = typer.Option(
        None,
        help="Framework used as the default result to fill missing values for the frameworks in `frameworks`. E.g., if `framework_nan_fill='foo'`, for dataset A of `framework='bar'` has no result (or NaN result), it is replaced by the result of `'foo'` on dataset A.",
    ),
    problem_types: Optional[List[str]] = typer.Option(
        None, help="List of problem types to filter results to. E.g. 'problem_type1,problem_type2,..."
    ),
    folds_to_keep: Optional[List[int]] = typer.Option(
        None, help="List of result folds to use. By default folds 0-9 (10-fold) are used."
    ),
    banned_datasets: Optional[List[str]] = typer.Option(
        None, help="List of datasets to skip during evaluation. E.g. 'dataset1,dataset2,..."
    ),
    infer_batch_size: int = typer.Option(
        None,
        help="If specified, will replace the `time_infer_s` column with the value in column `pred_time_test_with_transform_batch_size_{infer_batch_size}`. If a given row does not have a value in the infer_batch_size column, the original `time_infer_s` value is used.",
    ),
    compute_z_score: bool = typer.Option(
        True,
        help="Whether to compute comparison z-scores. The framework being compared against is the first one listed in `frameworks_run`. Only valid when multiple folds are used and `treat_folds_as_datasets=False`.",
    ),
    treat_folds_as_datasets: bool = typer.Option(False, help="If True, treat each fold as a separate dataset."),
    use_tid_as_dataset_name: bool = typer.Option(
        False,
        help="If True, replaces dataset human-readable names with unique integer IDs associated with their OpenML task ID.",
    ),
    filter_errors: bool = typer.Option(
        False,
        help="If True, any dataset missing a result from any framework in `frameworks` will be filtered from all results. Takes priority over `framework_nan_fill`.",
    ),
    clean_data: bool = typer.Option(
        True,
        help="If True, performs some general data cleanup to avoid inconsistencies in dataset naming compared to task ids.",
    ),
) -> Dict[str, pd.DataFrame]:
    """
    Generate evaluation results and saves the results to <results_dir>output/openml/<output_suffix>/evaluation_results.json

    Example:
    agbench evaluate-amlb-results --frameworks_run framework_1 --frameworks_run framework_2 --paths openml_ag_ag_bench_20230707T070230.csv --results-dir-input data/results/input/prepared/openml --no-clean-data
    """
    evaluate(
        frameworks_run=frameworks_run if frameworks_run else None,
        paths=paths if paths else None,
        results_dir=results_dir,
        results_dir_input=results_dir_input,
        results_dir_output=results_dir_output,
        output_suffix=output_suffix,
        framework_nan_fill=framework_nan_fill,
        problem_type=problem_types if problem_types else None,
        folds_to_keep=folds_to_keep if folds_to_keep else None,
        compute_z_score=compute_z_score,
        treat_folds_as_datasets=treat_folds_as_datasets,
        banned_datasets=banned_datasets if banned_datasets else None,
        infer_batch_size=infer_batch_size,
        clean_data=clean_data,
        use_tid_as_dataset_name=use_tid_as_dataset_name,
        filter_errors=filter_errors,
    )


# TODO: Rename to a more description function, or convert to a class
def evaluate(
    *,
    paths: List[str] | pd.DataFrame,
    frameworks_run: List[str] | None = None,
    frameworks_compare_vs_all: List[str] | str | None = "auto",
    results_dir: str = "data/results/",
    results_dir_input: str = None,
    results_dir_output: str = None,
    output_suffix: str | None = None,
    frameworks_rename: dict | None = None,
    framework_nan_fill: str | None = None,
    worst_nan_fill: bool = False,
    problem_type: List[str] | str | None = None,
    folds_to_keep: List[int] | None = None,
    compute_z_score: bool = True,
    treat_folds_as_datasets: bool = False,
    banned_datasets: List[str] | None = None,
    infer_batch_size: int | None = None,
    clean_data: bool = True,
    use_tid_as_dataset_name: bool = False,
    filter_errors: bool = False,
    task_metadata: str = None,
    verbose: bool = True,
) -> (pd.DataFrame, pd.DataFrame, pd.DataFrame, pd.DataFrame, Dict[str, pd.DataFrame]):
    """
    # TODO: Add description
    Parameters
    ----------
    frameworks_run : List[str]
        The list of frameworks to compare.
        These frameworks must be present in the "frameworks" column of the loaded input files listed in the `paths` arg.
    frameworks_compare_vs_all : List[str] | str | None, default = "auto"
        List of frameworks to compare all other frameworks to in a dedicated table.
        If "auto" sets to [frameworks_run[0]].
    paths : List[str]
        The list of file paths to load the input data from.
        The resulting input DataFrame will be the concatenation of all files listed in `paths`.
        Filepaths can include files located in s3 assuming you have the proper read permissions.
        # TODO: Define each column that must exist in paths
        # TODO: Consider allowing non-preprocessed data that is preprocessed on-the-fly for convenience
        # TODO: Allow passing the raw results DataFrame directly rather than requiring always loading from paths.
    output_suffix : str
        The output suffix of the path to save the output files.
        # TODO: Expand this description, add more control over save locations, add option to avoid saving entirely
    framework_nan_fill : str, optional
        The framework used as the default result to fill missing values for the frameworks in `frameworks_run`.
        For example, if `framework_nan_fill='foo'`, for dataset A if `framework='bar'` has no result (or NaN result),
        it is replaced by the result of `'foo'` on dataset A.
        A common usage of this is to specify some simple baseline, such as a constant predictor or dummy model.
    problem_type : List[str] | str, optional, default None
        The list of problem types to filter results to. By default, all problem types are used, and no filtering occurs.
    folds_to_keep : List[int], optional
        The list of result folds to use. All others are dropped. By default folds 0-9 (10-fold) are used.
        More folds leads to greater statistical certainty.
    compute_z_score : bool, default True
        Whether to compute comparison z-scores. The framework being compared against is the first one listed in `frameworks_run`.
        Only valid when multiple folds are used and `treat_folds_as_datasets=False`.
    treat_folds_as_datasets : bool, default False
        If True, all dataset x fold results are treated as their own separate datasets,
        rather than averaging the fold results together.
    banned_datasets : List[str], optional
        If specified, the list of datasets are filtered from the datasets used in evaluation.
    infer_batch_size : int, optional
        If specified, will replace the `time_infer_s` column with the value in column `pred_time_test_with_transform_batch_size_{infer_batch_size}`
        If a given row does not have a value in the infer_batch_size column, the original `time_infer_s` value is used.
    clean_data : bool, default True
        If True, performs some general data cleanup to avoid inconsistencies in dataset naming compared to task ids.
        If True, `time_infer_s` is updated to be the time per row, rather than the overall time.
        Must be set to False if using results that don't contain tids or lack a task metadata file.
    use_tid_as_dataset_name : bool, default False
        If True, replaces dataset human-readable names with unique integer IDs associated with their OpenML task ID.
    filter_errors : bool, default False
        If True, any dataset missing a result from any framework in `frameworks_run` will be filtered from all results.
        This ensures a dense result evaluation, but may lead to many datasets being filtered due to sparse framework failures.
        If True, takes priority over `framework_nan_fill`.
    Returns
    -------
    results_ranked : pd.DataFrame
        A pandas DataFrame with 1 row per framework in `frameworks_run`.
        Includes a variety of analytics of overall performance aggregated across datasets.
        This result only contains datasets where all frameworks succeeded.
        This is the aggregated form of `results_ranked_by_dataset`.
        Default column names:
        ['framework', 'time_train_s', 'metric_error', 'time_infer_s', 'bestdiff', 'loss_rescaled',
        'time_train_s_rescaled', 'time_infer_s_rescaled',
        'rank', 'rank=1_count', 'rank=2_count', 'rank=3_count', 'rank>3_count', 'error_count']
        Column Definitions
            framework : The name of the framework
            time_train_s : The mean training time in seconds
            time_infer_s : The mean inference time in seconds (per row if clean_data=True)
            bestdiff : The mean percentage relative less errors the best framework on a given dataset has compared to this framework.
                If the framework is the best for a given dataset, then the bestdiff value on that dataset is 0.
                If the best framework gets 0.05 metric_error, and framework FOO gets 0.20 metric_error,
                    then bestdiff for this dataset is 0.75 for FOO because (0.20 - 0.05)/0.20 = 0.75
                    (aka FOO has 4x more error than the best framework, and the best framework has 75% less error than FOO)
                0 is the best, 1 is the worst.
            loss_rescaled : The mean rescaled version of `metric_error`, where the best framework is rescaled to 0 and the worst framework is rescaled to 1 for each dataset.
                0 is the best, 1 is the worst.
            *_rescaled : The mean rescaled version of the original column, where the fastest framework for each dataset is rescaled to 1.
            rank : mean rank
            rank=1_count : number of undisputed first-place (champion) datasets
            rank=2_count : number of second-place datasets (or 2-way tie for 1st place) (rank >1, <=2)
            rank=3_count : number of third-place datasets (rank >2, <=3)
            rank>3_count : number of datasets with placement worse than 3rd.
            error_count : number of failed datasets with no result.
    results_ranked_all : pd.DataFrame
        Identical to `results_ranked`, except it also includes datasets where a subset of frameworks failed.
    results_ranked_by_dataset : pd.DataFrame
        A pandas DataFrame with N rows per framework, where N is the number of datasets.
        This result contains evaluations on a per-dataset granularity.
        `results_ranked` is the aggregated form of this result.
        # TODO: Define each column
    results_ranked_by_dataset_all : pd.DataFrame
        Identical to `results_ranked_by_dataset`, except it also includes datasets where a subset of frameworks failed.
    results_pairs_merged_dict : Dict[str, pd.DataFrame]
        For each framework that was pair-wise compared with other frameworks, the key is the framework name,
        and the value is a pandas DataFrame of the comparison evaluation results.
        In general, only the first framework in `frameworks_run` is pair-wise compared.
        # TODO: Define each column
    """
    if results_dir_input is None:
        results_dir_input = os.path.join(results_dir, "input/prepared/openml/")
    if results_dir_output is None:
        results_dir_output = os.path.join(results_dir, f"output/openml/{output_suffix}/")

    benchmark_evaluator = BenchmarkEvaluator(
        results_dir_input=results_dir_input,
        results_dir_output=results_dir_output,
        output_suffix=output_suffix,
        use_tid_as_dataset_name=use_tid_as_dataset_name,
        framework_nan_fill=framework_nan_fill,
        worst_nan_fill=worst_nan_fill,
        filter_errors=filter_errors,
        task_metadata=task_metadata,
    )

    results_raw = benchmark_evaluator.load_data(
        paths=paths,
        frameworks=frameworks_run,
        folds=folds_to_keep,
        problem_type=problem_type,
        banned_datasets=banned_datasets,
        infer_batch_size=infer_batch_size,
        treat_folds_as_datasets=treat_folds_as_datasets,
        clean_data=clean_data,
    )

    if frameworks_run is None:
        frameworks_run = sorted(list(results_raw["framework"].unique()))

    if frameworks_rename is not None:
        results_raw["framework"] = results_raw["framework"].map(frameworks_rename).fillna(results_raw["framework"])
        frameworks_run = [frameworks_rename.get(f, f) for f in frameworks_run]

    if isinstance(frameworks_compare_vs_all, str) and frameworks_compare_vs_all == "auto":
        frameworks_compare_vs_all = [frameworks_run[0]]
    elif frameworks_compare_vs_all is None:
        frameworks_compare_vs_all = []
    if not isinstance(frameworks_compare_vs_all, list):
        raise ValueError(f"Unexpected value for frameworks_compare_vs_all: {frameworks_compare_vs_all}")

    if verbose:
        print("frameworks = [")
        for i in range(len(frameworks_run)):
            print(f'\t"{frameworks_run[i]}",')
        print("]")

    folds_to_keep = sorted(results_raw["fold"].unique())

    if len(frameworks_run) > 1:
        win_rate_per_dataset_df = compute_win_rate_per_dataset(
            f1=frameworks_run[0],
            f2=frameworks_run[1],
            results_raw=results_raw,
            folds=folds_to_keep,
            verbose=verbose,
        )
    if compute_z_score and len(frameworks_run) > 1 and len(folds_to_keep) > 1:
        z_stat_df = compute_stderr_z_stat_bulk(
            framework=frameworks_run[0],
            frameworks_to_compare=frameworks_run[1:],
            results_raw=results_raw,
            verbose=verbose,
        )
        z_stat_series = compute_stderr_z_stat(
            results_raw, f1=frameworks_run[0], f2=frameworks_run[1], folds=folds_to_keep, verbose=False
        )
        if verbose:
            graph_vs(results_df=results_raw, f1=frameworks_run[0], f2=frameworks_run[1], z_stats=z_stat_series)

    (
        results_ranked,
        results_ranked_by_dataset,
        results_ranked_all,
        results_ranked_by_dataset_all,
        results_pairs_merged_dict,
    ) = evaluate_results.evaluate(
        results_raw=results_raw,
        frameworks=frameworks_run,
        columns_to_agg_extra=[
            TIME_INFER_S,
        ],
        frameworks_compare_vs_all=frameworks_compare_vs_all,
        output_dir=benchmark_evaluator.results_dir_output,
        verbose=verbose,
    )

    if treat_folds_as_datasets:
        results_ranked_by_dataset = convert_to_dataset_fold(results_ranked_by_dataset)
        results_ranked_by_dataset_all = convert_to_dataset_fold(results_ranked_by_dataset_all)

    return (
        results_ranked,
        results_ranked_by_dataset,
        results_ranked_all,
        results_ranked_by_dataset_all,
        results_pairs_merged_dict,
    )


if __name__ == "__main__":
    app()



import os
import unittest
from unittest.mock import patch

from autogluon.bench.utils.dataset_utils import get_data_home_dir, get_home_dir, get_repo_url


class TestUtils(unittest.TestCase):
    def test_get_home_dir(self):
        expected_path = os.path.expanduser("~/.autogluon_bench")
        self.assertEqual(get_home_dir(), expected_path)

    def test_get_data_home_dir(self):
        with patch("autogluon.bench.utils.dataset_utils.get_home_dir", return_value="/home/user"):
            expected_path = "/home/user/datasets"
            self.assertEqual(get_data_home_dir(), expected_path)

    def test_get_repo_url_default(self):
        expected_url = "https://automl-mm-bench.s3.amazonaws.com/"
        self.assertEqual(get_repo_url(), expected_url)

    def test_get_repo_url_custom(self):
        custom_url = "https://custom-repo-url.com"
        expected_url = custom_url + "/"
        with patch.dict("os.environ", {"AUTOGLUON_BENCH_REPO": custom_url}):
            self.assertEqual(get_repo_url(), expected_url)



#!/usr/bin/env python3

import os

import aws_cdk as cdk
from aws_cdk import aws_iam as iam
from aws_cdk import aws_lambda as _lambda
from constructs import Construct


class BatchLambdaFunction(Construct):
    """
    Custom CDK construct class for a Lambda function with invocation
    rights for AWS Batch.
    """

    @property
    def function(self):
        return self._lambda_function

    def __init__(
        self,
        scope: Construct,
        id: str,
        prefix: str,
        tag: str,
        function_name: str,
        code_path: str,
        environment,
        timeout=600,
    ):
        super().__init__(scope, id)
        aws_account_region = os.environ["CDK_DEPLOY_REGION"]
        aws_account_id = os.environ["CDK_DEPLOY_ACCOUNT"]

        self._lambda_function_role = iam.Role(
            self,
            f"{prefix}-lambda-role",
            assumed_by=iam.ServicePrincipal("lambda.amazonaws.com"),
            managed_policies=[
                iam.ManagedPolicy.from_aws_managed_policy_name("service-role/AWSLambdaBasicExecutionRole")
            ],
            inline_policies={
                "SubmitBatchForLambda": iam.PolicyDocument(
                    statements=[
                        iam.PolicyStatement(
                            resources=[
                                f"arn:aws:batch:{aws_account_region}:{aws_account_id}:job-definition/*",
                                f"arn:aws:batch:{aws_account_region}:{aws_account_id}:job-queue/*",
                            ],
                            conditions={
                                "StringEquals": {
                                    f"aws:ResourceTag/{prefix}": tag  # resources with tag, e.g. {"automm": "vision"}
                                }
                            },
                            actions=["batch:SubmitJob"],
                        )
                    ]
                )
            },
        )

        self._lambda_function = _lambda.Function(
            self,
            f"{prefix}-lambda",
            function_name=function_name,
            runtime=_lambda.Runtime.PYTHON_3_8,
            environment=environment,
            code=_lambda.Code.from_asset(
                code_path,
                bundling=cdk.BundlingOptions(
                    image=_lambda.Runtime.PYTHON_3_8.bundling_image,
                    command=[
                        "bash",
                        "-c",
                        "pip install --no-cache -r requirements.txt -t /asset-output && cp -au . /asset-output",
                    ],
                ),
            ),
            handler="lambda_function.handler",
            timeout=cdk.Duration.seconds(timeout),
            role=self._lambda_function_role,
        )



import os
from unittest.mock import patch

import aws_cdk.aws_ec2 as ec2
import aws_cdk.aws_s3 as s3
from aws_cdk import App, Stack
from aws_cdk.aws_batch import CfnComputeEnvironment, CfnJobDefinition, CfnJobQueue
from aws_cdk.aws_ec2 import CfnLaunchTemplate
from aws_cdk.aws_ecr_assets import DockerImageAsset
from aws_cdk.aws_iam import CfnInstanceProfile, Role
from conftest import context_values, env

from autogluon.bench.cloud.aws.batch_stack.constructs.batch_lambda_function import BatchLambdaFunction
from autogluon.bench.cloud.aws.batch_stack.stack import BatchJobStack, StaticResourceStack


def test_static_resource_stack_without_vpc():
    app = App()
    for key, value in context_values.items():
        app.node.set_context(key, value)

    with (
        patch.object(
            StaticResourceStack,
            "create_s3_resources",
            return_value=s3.Bucket(Stack(app, "TestBucketStack"), "DummyBucket"),
        ) as mock_s3_resources,
        patch.dict(os.environ, {"CDK_DEPLOY_REGION": "dummy_region"}),
    ):
        stack = StaticResourceStack(app, "TestStaticResourceStack", env=env)

        mock_s3_resources.assert_called_once()

        assert stack.vpc is None


def test_static_resource_stack_with_vpc():
    app = App()
    for key, value in context_values.items():
        app.node.set_context(key, value)
    app.node.set_context("VPC_NAME", "ProvidedVpcName")

    with (
        patch.object(
            StaticResourceStack,
            "create_s3_resources",
            return_value=s3.Bucket(Stack(app, "TestBucketStack"), "DummyBucket"),
        ) as mock_s3_resources,
        patch.dict(os.environ, {"CDK_DEPLOY_REGION": "dummy_region"}),
    ):
        stack = StaticResourceStack(app, "TestStaticResourceStack", env=env)

        mock_s3_resources.assert_called_once()

        assert stack.vpc is not None


@patch.dict("os.environ", {"CDK_DEPLOY_REGION": "dummy_region", "CDK_DEPLOY_ACCOUNT": "dummy_account"}, clear=True)
def test_batch_job_stack():
    app = App()
    for key, value in context_values.items():
        app.node.set_context(key, value)

    os.environ["AG_BENCH_BASE_IMAGE"] = "dummy_base_image"
    os.environ["FRAMEWORK_PATH"] = "frameworks/tabular"
    os.environ["GIT_URI"] = "https://github.com/openml/automlbenchmark.git"
    os.environ["GIT_BRANCH"] = "master"

    with patch.object(
        StaticResourceStack,
        "create_s3_resources",
        return_value=s3.Bucket(Stack(app, "TestBucketStack"), "DummyBucket"),
    ):
        static_resource_stack = StaticResourceStack(app, "TestStaticResourceStack", env=env)

        assert static_resource_stack.vpc is None

        dummy_stack = Stack(app, "DummyVpcStack")
        static_resource_stack.metrics_bucket = s3.Bucket(dummy_stack, "DummyMetricsBucket")
        static_resource_stack.data_bucket = s3.Bucket(dummy_stack, "DummyDataBucket")

        batch_job_stack = BatchJobStack(app, "TestBatchJobStack", static_stack=static_resource_stack, env=env)
        prefix = app.node.try_get_context("STACK_NAME_PREFIX")
        lambda_function_name = app.node.try_get_context("LAMBDA_FUNCTION_NAME")
        constructs = [
            (f"{prefix}-security-group", ec2.SecurityGroup),
            (f"{prefix}-ecr-docker-image-asset", DockerImageAsset),
            (f"{prefix}-job-definition", CfnJobDefinition),
            (f"{prefix}-launch-template", CfnLaunchTemplate),
            (f"{prefix}-instance-role", Role),
            (f"{prefix}-instance-profile", CfnInstanceProfile),
            (f"{prefix}-compute-environment", CfnComputeEnvironment),
            (f"{prefix}-job-queue", CfnJobQueue),
            (f"{lambda_function_name}-{prefix}", BatchLambdaFunction),
            ("vpc", ec2.Vpc),
        ]

        for construct_id, construct_class in constructs:
            if construct_id == "vpc":
                constructs = [c for c in batch_job_stack.node.children if isinstance(c, ec2.Vpc)]
                assert constructs, "No VPC found in BatchJobStack"
                assert isinstance(
                    constructs[0], construct_class
                ), f"First VPC in BatchJobStack is not an instance of {construct_class.__name__}"
            else:
                construct = batch_job_stack.node.try_find_child(construct_id)
                assert construct is not None, f"{construct_id} not found"
                assert isinstance(
                    construct, construct_class
                ), f"{construct_id} is not an instance of {construct_class.__name__}"



import importlib.resources
import json
import os
import tempfile
from unittest.mock import mock_open, patch

import yaml
from conftest import default_configs

from autogluon.bench.cloud.aws.stack_handler import (
    _get_temp_cdk_app_path,
    construct_context,
    deploy_stack,
    destroy_stack,
)


def test_get_temp_cdk_app_path(mocker):
    mock_mkdtemp = mocker.patch("tempfile.mkdtemp")
    mock_copy2 = mocker.patch("shutil.copy2")
    mock_chmod = mocker.patch("os.chmod")
    mock_join = mocker.patch("os.path.join")
    mock_resources_path = mocker.patch("importlib.resources.path")

    mock_mkdtemp.return_value = "/temp/dir"
    mock_join.return_value = "/temp/dir/app.py"

    mock_resources_path.return_value.__enter__.return_value = "path_to_cdk_app"

    result = _get_temp_cdk_app_path()

    assert result == "/temp/dir/app.py"
    mock_mkdtemp.assert_called_once()
    mock_join.assert_called_once_with("/temp/dir", "app.py")
    mock_copy2.assert_called_once_with("path_to_cdk_app", "/temp/dir/app.py")
    mock_chmod.assert_called_once_with("/temp/dir/app.py", 0o755)


def test_construct_context():
    custom_configs = {"CDK_DEPLOY_ACCOUNT": "123456789012", "CDK_DEPLOY_REGION": "us-west-1", "PREFIX": "test"}

    with patch("builtins.open", mock_open(read_data=json.dumps(default_configs))) as mock_file:
        with patch("autogluon.bench.cloud.aws.stack_handler.get_instance_type_specs") as mock_instance_specs:
            mock_instance_specs.return_value = (1, 8, 32000)
            context = construct_context(custom_configs=custom_configs)

    prefix = custom_configs["PREFIX"]
    assert context["CDK_DEPLOY_ACCOUNT"] == custom_configs["CDK_DEPLOY_ACCOUNT"]
    assert context["CDK_DEPLOY_REGION"] == custom_configs["CDK_DEPLOY_REGION"]
    assert context["STACK_NAME_PREFIX"] == prefix
    assert context["METRICS_BUCKET"] == default_configs["METRICS_BUCKET"]
    assert context["DATA_BUCKET"] == default_configs["DATA_BUCKET"]
    assert context["VPC_NAME"] == default_configs["VPC_NAME"]
    assert context["STATIC_RESOURCE_STACK_NAME"] == f"{prefix}-static-resource-stack"
    assert context["BATCH_STACK_NAME"] == f"{prefix}-batch-stack"
    assert context["CONTAINER_GPU"] == 1
    assert context["CONTAINER_VCPU"] == 8
    assert context["CONTAINER_MEMORY"] == 22000
    assert context["COMPUTE_ENV_MAXV_CPUS"] == 16


def test_deploy_stack(mocker):
    infra_configs_dict = {
        "STACK_NAME_PREFIX": "test-prefix",
        "STACK_NAME_TAG": "benchmark",
        "STATIC_RESOURCE_STACK_NAME": "test-prefix-static-resource-stack",
        "BATCH_STACK_NAME": "test-prefix-batch-stack",
        "CONTAINER_MEMORY": 10000,
        "CDK_DEPLOY_REGION": "us-west-2",
        "INSTANCE_TYPES": ["m5.2xlarge"],
    }
    custom_configs = {
        "cdk_context": {
            "METRICS_BUCKET": "test-metrics-bucket",
            "DATA_BUCKET": "test-data-bucket",
        },
    }

    mock_subprocess = mocker.patch("subprocess.check_call")
    with importlib.resources.path("autogluon.bench.cloud.aws", "stack_handler.py") as file_path:
        module_base_dir = os.path.dirname(file_path)

    with tempfile.TemporaryDirectory() as temp_dir:
        with tempfile.NamedTemporaryFile(dir=temp_dir, delete=False) as temp_file:
            mock_cdk_path = mocker.patch(
                "autogluon.bench.cloud.aws.stack_handler._get_temp_cdk_app_path", return_value=temp_file.name
            )
            mock_get_context = mocker.patch(
                "autogluon.bench.cloud.aws.stack_handler.construct_context", return_value=infra_configs_dict
            )

            deploy_stack(custom_configs=custom_configs)

    assert not os.path.exists(temp_dir)
    mock_cdk_path.assert_called_once()
    mock_get_context.assert_called_with(custom_configs=custom_configs["cdk_context"])
    mock_subprocess.assert_called_once_with(
        [
            os.path.join(module_base_dir, "deploy.sh"),
            infra_configs_dict["STATIC_RESOURCE_STACK_NAME"],
            infra_configs_dict["BATCH_STACK_NAME"],
            temp_file.name,
        ]
    )


def test_destroy_stack(mocker):
    mock_subprocess = mocker.patch("subprocess.check_call")
    static_resource_stack = "static_resource_stack"
    batch_stack = "batch_stack"
    cdk_deploy_account = "cdk_deploy_account"
    cdk_deploy_region = "cdk_deploy_region"

    with importlib.resources.path("autogluon.bench.cloud.aws", "stack_handler.py") as file_path:
        module_base_dir = os.path.dirname(file_path)

    with tempfile.TemporaryDirectory() as temp_dir:
        with tempfile.NamedTemporaryFile(dir=temp_dir, delete=False) as temp_file:
            mock_cdk_path = mocker.patch(
                "autogluon.bench.cloud.aws.stack_handler._get_temp_cdk_app_path", return_value=temp_file.name
            )

            destroy_stack(
                static_resource_stack=static_resource_stack,
                batch_stack=batch_stack,
                cdk_deploy_account=cdk_deploy_account,
                cdk_deploy_region=cdk_deploy_region,
                config_file=None,
            )

    assert not os.path.exists(temp_dir)
    assert os.environ["CDK_DEPLOY_ACCOUNT"] == cdk_deploy_account
    assert os.environ["CDK_DEPLOY_REGION"] == cdk_deploy_region

    mock_cdk_path.assert_called_once()
    mock_subprocess.assert_called_once_with(
        [
            os.path.join(module_base_dir, "destroy.sh"),
            static_resource_stack,
            batch_stack,
            cdk_deploy_region,
            temp_file.name,
        ]
    )


def test_destroy_stack_with_config(mocker):
    mock_subprocess = mocker.patch("subprocess.check_call")
    configs = {
        "STATIC_RESOURCE_STACK_NAME": "static_resource_stack",
        "BATCH_STACK_NAME": "batch_stack",
        "CDK_DEPLOY_ACCOUNT": "cdk_deploy_account",
        "CDK_DEPLOY_REGION": "cdk_deploy_region",
    }

    with importlib.resources.path("autogluon.bench.cloud.aws", "stack_handler.py") as file_path:
        module_base_dir = os.path.dirname(file_path)

    with tempfile.TemporaryDirectory() as temp_dir:
        with tempfile.NamedTemporaryFile(dir=temp_dir, delete=False) as temp_file:
            mock_cdk_path = mocker.patch(
                "autogluon.bench.cloud.aws.stack_handler._get_temp_cdk_app_path", return_value=temp_file.name
            )
            config_file_path = os.path.join(temp_dir, "configs.yaml")
            with open(config_file_path, "w") as f:
                yaml.dump(configs, f)

            destroy_stack(
                static_resource_stack=None,
                batch_stack=None,
                cdk_deploy_account=None,
                cdk_deploy_region=None,
                config_file=config_file_path,
            )

    assert not os.path.exists(temp_dir)
    assert os.environ["CDK_DEPLOY_ACCOUNT"] == "cdk_deploy_account"
    assert os.environ["CDK_DEPLOY_REGION"] == "cdk_deploy_region"

    mock_cdk_path.assert_called_once()
    mock_subprocess.assert_called_once_with(
        [
            os.path.join(module_base_dir, "destroy.sh"),
            "static_resource_stack",
            "batch_stack",
            "cdk_deploy_region",
            temp_file.name,
        ]
    )



import os


def get_home_dir():
    """Get home directory"""
    home_dir = os.environ.get("AUTOGLUON_BENCH_HOME", os.path.join("~", ".autogluon_bench"))
    # expand ~ to actual path
    home_dir = os.path.expanduser(home_dir)
    return home_dir


def get_data_home_dir():
    """Get home directory for storing the datasets"""
    home_dir = get_home_dir()
    return os.path.join(home_dir, "datasets")


def get_repo_url():
    """Return the base URL for dataset repository"""
    default_repo = "https://automl-mm-bench.s3.amazonaws.com"
    repo_url = os.environ.get("AUTOGLUON_BENCH_REPO", default_repo)
    if repo_url[-1] != "/":
        repo_url = repo_url + "/"
    return repo_url


def path_expander(path, base_folder):
    path_l = path.split(";")
    return ";".join([os.path.abspath(os.path.join(base_folder, path)) for path in path_l])



import io
import os

import yaml

from autogluon.bench.runbenchmark import get_job_status, get_kwargs, invoke_lambda, run, run_benchmark


def setup_mock(
    mocker, tmp_path: str, module: str = "tabular", custom_dataloader: dict = None, custom_metrics: dict = None
):
    config_file = tmp_path / "run_configs.yaml"
    config_file.touch()
    # mock_open = mocker.patch("builtins.open", new_callable=mocker.mock_open)
    mocker.patch("re.search", return_value=False)
    cdk_context = {"METRICS_BUCKET": "test_bucket"}
    job_configs = {"job_id_1": "config_spit_1", "job_id_2": "config_spit_2"}

    infra_configs = {
        "STATIC_RESOURCE_STACK_NAME": "test_static_stack",
        "BATCH_STACK_NAME": "test_batch_stack",
        "CDK_DEPLOY_ACCOUNT": "test_account",
        "CDK_DEPLOY_REGION": "test_region",
        "METRICS_BUCKET": "test_bucket",
    }

    if module in ["tabular", "timeseries"]:
        module_configs = {
            "git_uri#branch": "https://github.com/openml/automlbenchmark.git#master",
            "framework": "AutoGluon:stable",
            "amlb_benchmark": "small",
            "amlb_user_dir": "sample_configs/amlb_configs",
        }
    elif module == "multimodal":
        module_configs = {
            "dataset_name": "data",
            "framework": "AutoGluon_stable",
            "custom_dataloader": custom_dataloader,
            "custom_metrics": custom_metrics,
        }

    yaml_value = {"mode": "aws", "module": module, "cdk_context": cdk_context, "job_configs": job_configs}
    yaml_value.update(infra_configs)
    yaml_value.update(module_configs)

    def mock_yaml_side_effect(file_obj):
        if hasattr(file_obj, "name"):
            if "run_configs" in file_obj.name:
                return yaml_value
            else:
                return original_safe_load(file_obj)
        return {}

    original_safe_load = yaml.safe_load
    mock_yaml = mocker.patch("yaml.safe_load", side_effect=mock_yaml_side_effect)
    mocker.patch("autogluon.bench.runbenchmark._get_benchmark_name", return_value="test_benchmark")
    mocker.patch("autogluon.bench.runbenchmark.formatted_time", return_value="test_time")
    mocker.patch("autogluon.bench.runbenchmark._dump_configs", return_value="test_dump")
    mocker.patch("autogluon.bench.runbenchmark._dump_configs", return_value="test_dump")
    mocker.patch("os.environ.__setitem__")

    mock_deploy_stack = mocker.patch("autogluon.bench.runbenchmark.deploy_stack", return_value=infra_configs)
    mock_upload_to_s3 = mocker.patch("autogluon.bench.runbenchmark.upload_to_s3", return_value="test_s3_path")
    mock_invoke_lambda = mocker.patch("autogluon.bench.runbenchmark.invoke_lambda", return_value={})

    mock_wait_for_jobs = mocker.patch("autogluon.bench.runbenchmark.wait_for_jobs", return_value=[])
    mock_get_hardware_metrics = mocker.patch("autogluon.bench.runbenchmark.get_hardware_metrics")
    mock_destroy_stack = mocker.patch("autogluon.bench.runbenchmark.destroy_stack")
    mock_mount = mocker.patch("autogluon.bench.runbenchmark._mount_dir")
    mock_umount = mocker.patch("autogluon.bench.runbenchmark._umount_if_needed")

    return {
        "config_file": str(config_file),
        "infra_configs": infra_configs,
        "mock_deploy_stack": mock_deploy_stack,
        "mock_upload_to_s3": mock_upload_to_s3,
        "mock_invoke_lambda": mock_invoke_lambda,
        "mock_wait_for_jobs": mock_wait_for_jobs,
        "mock_destroy_stack": mock_destroy_stack,
        "mock_get_hardware_metrics": mock_get_hardware_metrics,
        "mock_mount": mock_mount,
        "mock_umount": mock_umount,
        "cdk_context": cdk_context,
        "job_configs": job_configs,
        "custom_configs": yaml_value,
    }


def test_get_kwargs_multimodal():
    module = "multimodal"
    configs = {"framework": "AutoGluon_stable", "dataset_name": "dataset"}

    expected_result = {
        "setup_kwargs": {"git_uri": "https://github.com/autogluon/autogluon.git", "git_branch": "stable"},
        "run_kwargs": {
            "dataset_name": "dataset",
            "framework": "AutoGluon_stable",
            "constraint": None,
            "params": {
                "predictor_args": {"presets": "best_quality"},
                "fit_args": {"hyperparameters": {"optimization.max_epochs": 10}},
            },
            "custom_dataloader": None,
            "custom_metrics": None,
            "time_limit": None,
        },
    }

    assert get_kwargs(module, configs) == expected_result


def test_get_kwargs_tabular():
    module = "tabular"
    configs = {
        "git_uri#branch": "https://github.com/openml/automlbenchmark.git#stable",
        "framework": "AutoGluon:stable",
        "amlb_benchmark": "test_bench",
        "amlb_task": "iris",
        "amlb_constraint": "test_constraint",
        "fold_to_run": 6,
        "amlb_user_dir": "sample_configs/amlb_configs",
    }

    expected_result = {
        "setup_kwargs": {
            "git_uri": "https://github.com/openml/automlbenchmark.git",
            "git_branch": "stable",
            "framework": "AutoGluon:stable",
            "user_dir": "sample_configs/amlb_configs",
        },
        "run_kwargs": {
            "framework": "AutoGluon:stable",
            "benchmark": "test_bench",
            "constraint": "test_constraint",
            "task": "iris",
            "fold": 6,
            "user_dir": "sample_configs/amlb_configs",
        },
    }

    assert get_kwargs(module, configs) == expected_result


def test_invoke_lambda(mocker):
    configs = {
        "CDK_DEPLOY_REGION": "us-east-1",
        "LAMBDA_FUNCTION_NAME": "test_function",
        "STACK_NAME_PREFIX": "prefix",
    }
    config_file = "test_config_file"

    mock_response = {"StatusCode": 200, "Payload": io.BytesIO(b'{"key": "value"}')}
    mock_lambda_client = mocker.MagicMock()
    mock_lambda_client.invoke.return_value = mock_response

    mocker.patch("boto3.client", return_value=mock_lambda_client)
    invoke_lambda(configs, config_file)
    mock_lambda_client.invoke.assert_called_once_with(
        FunctionName=configs["LAMBDA_FUNCTION_NAME"] + "-" + configs["STACK_NAME_PREFIX"],
        InvocationType="RequestResponse",
        Payload='{"config_file": "test_config_file"}',
    )


def test_run_aws_mode(mocker, tmp_path):
    setup = setup_mock(mocker, tmp_path)

    run(setup["config_file"], remove_resources=False, wait=False, skip_setup=True, save_hardware_metrics=False)

    setup["mock_deploy_stack"].assert_called_once_with(custom_configs=setup["custom_configs"])
    setup["mock_upload_to_s3"].assert_called_once_with(
        s3_bucket="test_bucket", s3_dir="configs/tabular/test_benchmark_test_time", local_path="test_dump"
    )
    setup["mock_invoke_lambda"].assert_called_once_with(configs=setup["infra_configs"], config_file="test_s3_path")
    setup["mock_wait_for_jobs"].assert_not_called()
    setup["mock_get_hardware_metrics"].assert_not_called(),
    setup["mock_destroy_stack"].assert_not_called()


def test_run_aws_mode_remove_resources(mocker, tmp_path):
    setup = setup_mock(mocker, tmp_path)

    run(setup["config_file"], remove_resources=True, wait=False, skip_setup=True, save_hardware_metrics=False)

    setup["mock_deploy_stack"].assert_called_once_with(custom_configs=setup["custom_configs"])
    setup["mock_upload_to_s3"].assert_called_once_with(
        s3_bucket="test_bucket", s3_dir="configs/tabular/test_benchmark_test_time", local_path="test_dump"
    )
    setup["mock_invoke_lambda"].assert_called_once_with(configs=setup["infra_configs"], config_file="test_s3_path")

    setup["mock_wait_for_jobs"].assert_called_once_with(config_file="test_dump")
    setup["mock_destroy_stack"].assert_called_once_with(
        static_resource_stack=setup["infra_configs"]["STATIC_RESOURCE_STACK_NAME"],
        batch_stack=setup["infra_configs"]["BATCH_STACK_NAME"],
        cdk_deploy_account=setup["infra_configs"]["CDK_DEPLOY_ACCOUNT"],
        cdk_deploy_region=setup["infra_configs"]["CDK_DEPLOY_REGION"],
        config_file=None,
    )


def test_run_aws_mode_wait(mocker, tmp_path):
    setup = setup_mock(mocker, tmp_path)

    run(setup["config_file"], remove_resources=False, wait=True, skip_setup=True, save_hardware_metrics=False)

    setup["mock_deploy_stack"].assert_called_once_with(custom_configs=setup["custom_configs"])
    setup["mock_upload_to_s3"].assert_called_once_with(
        s3_bucket="test_bucket", s3_dir="configs/tabular/test_benchmark_test_time", local_path="test_dump"
    )
    setup["mock_invoke_lambda"].assert_called_once_with(configs=setup["infra_configs"], config_file="test_s3_path")

    setup["mock_wait_for_jobs"].assert_called_once_with(config_file="test_dump")
    setup["mock_get_hardware_metrics"].assert_not_called()


def test_run_aws_tabular_user_dir(mocker, tmp_path):
    setup = setup_mock(mocker, tmp_path)
    temp_dir_mock = mocker.patch("tempfile.TemporaryDirectory")
    s3_mock = mocker.patch("autogluon.bench.utils.general_utils.download_dir_from_s3")

    run(setup["config_file"], remove_resources=False, wait=False, skip_setup=True, save_hardware_metrics=False)
    assert os.environ["FRAMEWORK_PATH"] == "frameworks/tabular"
    assert os.environ["GIT_URI"] == "https://github.com/openml/automlbenchmark.git"
    assert os.environ["GIT_BRANCH"] == "master"
    assert os.environ["AMLB_FRAMEWORK"] == "AutoGluon:stable"
    assert os.environ["AMLB_USER_DIR"] == "custom_configs/amlb_configs"
    temp_dir_mock.assert_not_called()
    s3_mock.assert_not_called()
    assert setup["mock_umount"].call_count == 4
    assert setup["mock_mount"].call_count == 2


def test_run_aws_multimodal_custom_dataloader(mocker, tmp_path):
    custom_dataloader = {
        "dataloader_file": "path_to/dataset.py",
        "class_name": "CustomDataset",
        "dataset_config_file": "path_to/datasets.yaml",
    }
    setup = setup_mock(mocker, tmp_path, module="multimodal", custom_dataloader=custom_dataloader)
    mount_mock = mocker.patch("autogluon.bench.runbenchmark._mount_dir")
    umount_mock = mocker.patch("autogluon.bench.runbenchmark._umount_if_needed")

    run(setup["config_file"], remove_resources=False, wait=False, skip_setup=True, save_hardware_metrics=False)
    assert setup["custom_configs"]["custom_dataloader"]["dataloader_file"] == "custom_configs/dataloaders/dataset.py"
    assert (
        setup["custom_configs"]["custom_dataloader"]["dataset_config_file"]
        == "custom_configs/dataloaders/datasets.yaml"
    )
    assert umount_mock.call_count == 2
    assert mount_mock.call_count == 1


def test_run_aws_multimodal_custom_metrics(mocker, tmp_path):
    custom_metrics = {"metrics_path": "path_to/metrics.py", "func_name": "custom_score"}
    setup = setup_mock(mocker, tmp_path, module="multimodal", custom_metrics=custom_metrics)
    mount_mock = mocker.patch("autogluon.bench.runbenchmark._mount_dir")
    umount_mock = mocker.patch("autogluon.bench.runbenchmark._umount_if_needed")

    run(setup["config_file"], remove_resources=False, wait=False, skip_setup=True, save_hardware_metrics=False)
    assert setup["custom_configs"]["custom_metrics"]["metrics_path"] == "custom_configs/metrics/metrics.py"
    assert umount_mock.call_count == 2
    assert mount_mock.call_count == 1


def test_run_local_mode(mocker, tmp_path):
    config_file = tmp_path / "config_split_test.yaml"
    config_file.touch()
    mock_open = mocker.patch("builtins.open", new_callable=mocker.mock_open)

    configs = {"mode": "local", "metrics_bucket": "test_bucket", "module": "tabular"}
    mock_yaml = mocker.patch("yaml.safe_load")
    mock_yaml.return_value = configs
    mocker.patch("autogluon.bench.runbenchmark._get_benchmark_name", return_value="test_benchmark")
    mocker.patch("autogluon.bench.runbenchmark.formatted_time", return_value="test_time")
    mock_run_benchmark = mocker.patch("autogluon.bench.runbenchmark.run_benchmark")

    run(str(config_file), remove_resources=False, wait=False, skip_setup=False, save_hardware_metrics=False)

    mock_open.assert_called_with(str(config_file), "r")
    mock_run_benchmark.assert_called_with(
        benchmark_name="test_benchmark_test_time",
        benchmark_dir="ag_bench_runs/tabular/test_benchmark_test_time",
        configs=configs,
        benchmark_dir_s3="tabular/test_benchmark_test_time/test_benchmark_test_time_0",
        skip_setup=False,
    )


def test_run_benchmark(mocker):
    benchmark_name = "test_benchmark"
    benchmark_dir = "test_dir"
    configs = {"module": "tabular", "METRICS_BUCKET": "test_bucket", "mode": "local"}

    mocker.patch(
        "autogluon.bench.runbenchmark.get_kwargs",
        return_value={
            "setup_kwargs": {},
            "run_kwargs": {
                "framework": "AutoGluon:stable",
                "benchmark": "toy",
                "constraint": "test_constraint",
                "task": "dummy_task",
                "fold": 0,
            },
        },
    )
    setup_mock = mocker.patch("autogluon.bench.runbenchmark.TabularBenchmark.setup")
    run_mock = mocker.patch("autogluon.bench.runbenchmark.TabularBenchmark.run")
    upload_metrics_mock = mocker.patch("autogluon.bench.runbenchmark.TabularBenchmark.upload_metrics")
    mocker.patch("autogluon.bench.runbenchmark._dump_configs")

    run_benchmark(
        benchmark_name=benchmark_name, benchmark_dir=benchmark_dir, configs=configs, benchmark_dir_s3="s3_dir"
    )

    setup_mock.assert_called_once_with()
    run_mock.assert_called_once_with(
        framework="AutoGluon:stable", benchmark="toy", constraint="test_constraint", task="dummy_task", fold=0
    )
    upload_metrics_mock.assert_called_once_with(
        s3_bucket=configs["METRICS_BUCKET"], s3_dir="s3_dir/local.toy.test_constraint.dummy_task.0.AutoGluon"
    )


def test_run_benchmark_skip_setup(mocker):
    benchmark_name = "test_benchmark"
    benchmark_dir = "test_dir"
    configs = {"module": "tabular", "METRICS_BUCKET": "test_bucket", "mode": "local"}

    mocker.patch(
        "autogluon.bench.runbenchmark.get_kwargs",
        return_value={
            "setup_kwargs": {},
            "run_kwargs": {
                "framework": "AutoGluon:stable",
                "benchmark": "toy",
                "constraint": "test_constraint",
                "task": "dummy_task",
                "fold": 0,
            },
        },
    )
    setup_mock = mocker.patch("autogluon.bench.runbenchmark.TabularBenchmark.setup")
    run_mock = mocker.patch("autogluon.bench.runbenchmark.TabularBenchmark.run")
    upload_metrics_mock = mocker.patch("autogluon.bench.runbenchmark.TabularBenchmark.upload_metrics")
    mocker.patch("autogluon.bench.runbenchmark._dump_configs")

    run_benchmark(
        benchmark_name=benchmark_name,
        benchmark_dir=benchmark_dir,
        configs=configs,
        benchmark_dir_s3="s3_dir",
        skip_setup=True,
    )

    setup_mock.assert_not_called()
    run_mock.assert_called_once_with(
        framework="AutoGluon:stable", benchmark="toy", constraint="test_constraint", task="dummy_task", fold=0
    )
    upload_metrics_mock.assert_called_once_with(
        s3_bucket=configs["METRICS_BUCKET"], s3_dir="s3_dir/local.toy.test_constraint.dummy_task.0.AutoGluon"
    )


def test_get_job_status_with_config_file(mocker, tmp_path):
    # Setup mock
    setup = setup_mock(mocker, tmp_path)

    # Additional mock for describe_jobs
    mock_boto_client = mocker.patch("boto3.client")
    mock_boto_client.return_value.describe_jobs.side_effect = [
        {"jobs": [{"status": "SUCCEEDED"}]},
        {"jobs": [{"status": "FAILED"}]},
    ]

    expected_status_dict = {"job_id_1": "SUCCEEDED", "job_id_2": "FAILED"}
    actual_status_dict = get_job_status(config_file=setup["config_file"])

    mock_boto_client.assert_called_once_with("batch", region_name="test_region")
    assert actual_status_dict == expected_status_dict


def test_get_job_status_with_job_ids(mocker, tmp_path):
    # Setup mock
    setup = setup_mock(mocker, tmp_path)

    # Additional mock for describe_jobs
    mock_boto_client = mocker.patch("boto3.client")
    mock_boto_client.return_value.describe_jobs.side_effect = [
        {"jobs": [{"status": "SUCCEEDED"}]},
        {"jobs": [{"status": "FAILED"}]},
    ]

    expected_status_dict = {"job_id_1": "SUCCEEDED", "job_id_2": "FAILED"}
    actual_status_dict = get_job_status(
        job_ids=["job_id_1", "job_id_2"], cdk_deploy_region="test_region", config_file=None
    )

    mock_boto_client.assert_called_once_with("batch", region_name="test_region")
    assert actual_status_dict == expected_status_dict


def test_get_kwargs_timeseries():
    module = "timeseries"
    configs = {
        "git_uri#branch": "https://github.com/openml/automlbenchmark.git#stable",
        "framework": "AutoGluon:stable",
        "amlb_benchmark": "test_bench",
        "amlb_task": "iris",
        "amlb_constraint": "test_constraint",
        "fold_to_run": 6,
        "amlb_user_dir": "sample_configs/amlb_configs",
    }

    expected_result = {
        "setup_kwargs": {
            "git_uri": "https://github.com/openml/automlbenchmark.git",
            "git_branch": "stable",
            "framework": "AutoGluon:stable",
            "user_dir": "sample_configs/amlb_configs",
        },
        "run_kwargs": {
            "framework": "AutoGluon:stable",
            "benchmark": "test_bench",
            "constraint": "test_constraint",
            "task": "iris",
            "fold": 6,
            "user_dir": "sample_configs/amlb_configs",
        },
    }

    assert get_kwargs(module, configs) == expected_result


def test_run_aws_timeseries_user_dir(mocker, tmp_path):
    setup = setup_mock(mocker, tmp_path, module="timeseries")
    temp_dir_mock = mocker.patch("tempfile.TemporaryDirectory")
    s3_mock = mocker.patch("autogluon.bench.utils.general_utils.download_dir_from_s3")

    run(setup["config_file"], remove_resources=False, wait=False, skip_setup=True)
    assert os.environ["FRAMEWORK_PATH"] == "frameworks/timeseries"
    assert os.environ["GIT_URI"] == "https://github.com/openml/automlbenchmark.git"
    assert os.environ["GIT_BRANCH"] == "master"
    assert os.environ["AMLB_FRAMEWORK"] == "AutoGluon:stable"
    assert os.environ["AMLB_USER_DIR"] == "custom_configs/amlb_configs"
    temp_dir_mock.assert_not_called()
    s3_mock.assert_not_called()
    assert setup["mock_umount"].call_count == 4
    assert setup["mock_mount"].call_count == 2


def test_run_benchmark_timeseries(mocker):
    benchmark_name = "test_benchmark"
    benchmark_dir = "test_dir"
    configs = {"module": "timeseries", "METRICS_BUCKET": "test_bucket", "mode": "local"}

    mocker.patch(
        "autogluon.bench.runbenchmark.get_kwargs",
        return_value={
            "setup_kwargs": {},
            "run_kwargs": {
                "framework": "AutoGluon:stable",
                "benchmark": "toy",
                "constraint": "test_constraint",
                "task": "dummy_task",
                "fold": 0,
            },
        },
    )
    setup_mock = mocker.patch("autogluon.bench.runbenchmark.TimeSeriesBenchmark.setup")
    run_mock = mocker.patch("autogluon.bench.runbenchmark.TimeSeriesBenchmark.run")
    upload_metrics_mock = mocker.patch("autogluon.bench.runbenchmark.TimeSeriesBenchmark.upload_metrics")
    mocker.patch("autogluon.bench.runbenchmark._dump_configs")

    run_benchmark(
        benchmark_name=benchmark_name, benchmark_dir=benchmark_dir, configs=configs, benchmark_dir_s3="s3_dir"
    )

    setup_mock.assert_called_once_with()
    run_mock.assert_called_once_with(
        framework="AutoGluon:stable", benchmark="toy", constraint="test_constraint", task="dummy_task", fold=0
    )
    upload_metrics_mock.assert_called_once_with(
        s3_bucket=configs["METRICS_BUCKET"], s3_dir="s3_dir/local.toy.test_constraint.dummy_task.0.AutoGluon"
    )



TASK = "task"
FRAMEWORK = "framework"
RESULT = "result"
DURATION = "duration"
PREDICT_DURATION = "predict_duration"

TIME_TRAIN_S = "time_train_s"
TIME_INFER_S = "time_infer_s"

DATASET = "dataset"
FOLD = "fold"
RANK = "rank"

METRIC_ERROR = "metric_error"
METRIC_SCORE = "metric_score"
METRIC = "metric"
PROBLEM_TYPE = "problem_type"

NUM_CLASSES = "num_classes"
NUM_COLS = "num_cols"
NUM_ROWS = "num_rows"
DATETIME = "datetime"

ERROR_COUNT = "error_count"
RANK_1 = "rank=1_count"

BESTDIFF = "bestdiff"

LOSS_RESCALED = "loss_rescaled"



from __future__ import annotations

import os
import warnings
from collections import defaultdict
from typing import List, Optional

import pandas as pd

from autogluon.common.loaders import load_pd
from autogluon.common.utils.s3_utils import is_s3_url

from .constants import DATASET, FOLD, FRAMEWORK, METRIC, METRIC_ERROR, PROBLEM_TYPE, TIME_INFER_S, TIME_TRAIN_S
from .metadata.metadata_loader import load_task_metadata
from .preprocess.preprocess_utils import (
    assert_unique_dataset_tid_pairs,
    convert_folds_into_separate_datasets,
    fill_missing_results_with_default,
    fill_missing_results_with_worst,
)


class BenchmarkEvaluator:
    def __init__(
        self,
        results_dir: str = "data/results/",
        results_dir_input: str = None,
        results_dir_output: str = None,
        output_suffix: str = "ag_full_v5/1h8c",
        use_tid_as_dataset_name: bool = False,
        filter_errors: bool = False,
        framework_nan_fill: Optional[str] = None,
        worst_nan_fill: bool = False,
        task_metadata: str | pd.DataFrame = "task_metadata.csv",
        filter_columns: bool = True,
        convert_infer_time_to_per_row: bool = True,
        columns_to_keep: Optional[List[str]] = None,
        columns_to_keep_extra: Optional[List[str]] = None,
        round_error_decimals: int | None = 5,
    ):
        """
        # TODO: Describe purpose of class
        # TODO: Add docstring for `load_data`
        # TODO: rename `results_raw` to be more descriptive

        Parameters
        ----------
        results_dir: str, default = 'data/results/'
            If results_dir_input or results_dir_output is absent,
                results_dir_input = results_dir + "input/prepared/openml/"
                results_dir_output = results_dir + f"output/openml/{output_suffix}/"
        results_dir_input: str, default = None
        results_dir_output: str, default = None
        output_suffix: str, default = 'ag_full_v5/1h8c'
            # TODO: Add docstring
        use_tid_as_dataset_name : bool, default = False
            If True, replaces `dataset` value with the `tid` value.
            Only valid when the results come from OpenML tasks.
            This is useful if `dataset` value is not globally unique but `tid` is,
            to avoid datasets with the same name being considered the same task.
        filter_errors : bool, default = False
            If True, any dataset that has a failure in any of the listed frameworks will be filtered out of all results.
                This means that there will be 0 errors for any framework in the returned results (dense representation).
            If False, datasets will not be filtered as long as they have at least 1 framework that did not fail on them.
        framework_nan_fill : Optional[str], default = None
            If specified, the value should refer to a framework used as the fill value for any other framework errors.
            If a framework had an error on a dataset,
            its result will be set to the result of the `framework_nan_fill` framework for that dataset.
            For example, if `framework_nan_fill='constantpredictor'`,
            framework dataset errors will be set to the `constantpredictor` framework result.
            This is aligned with how results were computed in the 2022 AMLB paper.
            This value is ignored if `filter_errors=True`, as the errors will already be filtered prior to this logic.
        worst_nan_fill : bool, default = False
            If True, will fill missing results with the worst result observed on a given task.
            Can be used as an alternative to `framework_nan_fill`.
            This value is ignored if `filter_errors=True`, as the errors will already be filtered prior to this logic.
        task_metadata: str | pd.DataFrame, default = 'task_metadata_289.csv'
            The path to task metadata file, or loaded pd.DataFrame of task metadata.
            This is only used when `clean_data=True` when calling `self.load_data`.
            This is used to filter to `tid` in `task_metadata` and join to get additional columns.
            This ensures that only datasets present in `task_metadata` will be used for analysis.
        filter_columns : bool, default = True
            If True, uses `columns_to_keep` and `extra_columns_to_keep` to filter columns in the output of `load_data`.
        columns_to_keep : Optional[List[str]], default = None
            Columns to filter to if `filter_columns=True`.
            If None, will use the default columns:
                DATASET
                FOLD
                FRAMEWORK
                METRIC_ERROR
                METRIC
                PROBLEM_TYPE
                TIME_TRAIN_S
                TIME_INFER_S
        columns_to_keep_extra : Optional[List[str]], default = None
            Extra columns to filter to if `filter_columns=True`.
            Will be concatenated with `columns_to_keep` if specified.
            This is used as a convenience argument to avoid having to respecify all
            standard columns when adding a new output column.
        round_error_decimals : int | None, default = 5
            The number of decimal places to round the metric error to.
            For example, if round_error_decimals=5, then a metric error of 0.03485612 becomes 0.03486.
            This is useful for detecting ties among frameworks by avoiding issues with floating point precision.
        """
        if columns_to_keep is None:
            columns_to_keep = [
                DATASET,
                FOLD,
                FRAMEWORK,
                METRIC_ERROR,
                METRIC,
                PROBLEM_TYPE,
                TIME_TRAIN_S,
                TIME_INFER_S,
            ]
        if columns_to_keep_extra is None:
            columns_to_keep_extra = []

        columns_to_keep = columns_to_keep + columns_to_keep_extra
        columns_to_keep_unique = set(columns_to_keep)
        if len(columns_to_keep_unique) != len(columns_to_keep):
            col_count_dict = defaultdict(int)
            for c in columns_to_keep:
                col_count_dict[c] += 1
            raise ValueError(
                f"Columns cannot be listed multiple times across "
                f"columns_to_keep and extra_columns_to_keep!"
                f"\n\tcolumn counts:\n\t\t{col_count_dict}"
            )

        self.convert_infer_time_to_per_row = convert_infer_time_to_per_row
        self.results_dir = results_dir
        self.results_dir_input = (
            results_dir + "input/prepared/openml/" if results_dir_input is None else results_dir_input
        )
        self.results_dir_output = (
            results_dir + f"output/openml/{output_suffix}/" if results_dir_output is None else results_dir_output
        )
        self._use_tid_as_dataset_name = use_tid_as_dataset_name
        self._filter_errors = filter_errors
        self._task_metadata = task_metadata
        if self._filter_errors:
            framework_nan_fill = None
        self._framework_nan_fill = framework_nan_fill
        self._worst_nan_fill = worst_nan_fill
        if self._worst_nan_fill and self._framework_nan_fill:
            raise AssertionError(
                f"Only one of framework_nan_fill and worst_nan_fill can be enabled: "
                f"(framework_nan_fill={self._framework_nan_fill}, worst_nan_fill={self._worst_nan_fill})"
            )
        if filter_columns:
            self._columns_to_keep = columns_to_keep
        else:
            self._columns_to_keep = None
        self.round_error_decimals = round_error_decimals

    def _load_results(
        self, paths: list | pd.DataFrame, clean_data: bool = False, banned_datasets: list = None
    ) -> pd.DataFrame:
        """paths can be either a list of file paths or a pandas DataFrame"""
        if isinstance(paths, list):
            results_raw = self.load_results_raw(paths=paths)
        else:
            assert isinstance(paths, pd.DataFrame)
            results_raw = paths
        if TIME_INFER_S in results_raw:
            with warnings.catch_warnings():
                warnings.filterwarnings("ignore")
                results_raw.loc[results_raw[TIME_INFER_S] == 0, TIME_INFER_S] = 0.001
        if clean_data:
            # FIXME: This doesn't work on new tasks due to not comprehensive metadata
            results_raw = self._clean_data(results_raw=results_raw)
        if banned_datasets is not None:
            results_raw = results_raw[~results_raw[DATASET].isin(banned_datasets)]
        if self._use_tid_as_dataset_name:
            assert_unique_dataset_tid_pairs(results_raw=results_raw)
            results_raw[DATASET] = results_raw["tid"].astype(int).astype(str)
            if banned_datasets is not None:
                results_raw = results_raw[~results_raw[DATASET].isin(banned_datasets)]
        len_results = len(results_raw)
        results_raw = results_raw.drop_duplicates(subset=[FRAMEWORK, DATASET, FOLD, METRIC])
        len_results_post_drop = len(results_raw)
        if len_results != len_results_post_drop:
            print(
                f"Note: Duplicate results detected! Dropping {len_results - len_results_post_drop} duplicate rows"
                f" ({len_results} -> {len_results_post_drop})...",
            )
        self._check_results_valid(results_raw=results_raw)
        return results_raw

    def load_results_raw(self, paths: list) -> pd.DataFrame:
        paths = [path if is_s3_url(path) else self.results_dir_input + path for path in paths]
        return pd.concat([load_pd.load(path) for path in paths], ignore_index=True, sort=True)

    def _check_results_valid(self, results_raw: pd.DataFrame):
        self._assert_unique_metric(results_raw=results_raw)

        if results_raw[METRIC_ERROR].min() < 0:
            eps = -1 / 1e8
            num_negative = len(results_raw[results_raw[METRIC_ERROR] < 0])
            if results_raw[METRIC_ERROR].min() < eps:
                raise AssertionError(
                    f"{METRIC_ERROR} cannot be negative! There may be a bug. "
                    f"Found min value: {results_raw[METRIC_ERROR].min()}"
                )
            else:
                print(
                    f"WARNING: min {METRIC_ERROR} was found to be negative, but was higher than epsilon {eps}! "
                    f"({results_raw[METRIC_ERROR].min()}) {num_negative} rows had negative values! "
                    f"Setting all negative values to 0."
                )
                results_raw.loc[results_raw[METRIC_ERROR] < 0, METRIC_ERROR] = 0

    def load_data(
        self,
        paths: list,
        frameworks: list = None,
        folds: list = None,
        clean_data: bool = False,
        problem_type=None,
        valid_datasets: list = None,
        banned_datasets: list = None,
        infer_batch_size: int = None,
        treat_folds_as_datasets: bool = False,
    ) -> pd.DataFrame:
        results_raw = self._load_results(paths=paths, clean_data=clean_data, banned_datasets=banned_datasets)
        if folds is not None:
            results_raw = results_raw[results_raw[FOLD].isin(folds)]
        if problem_type is not None:
            if isinstance(problem_type, list):
                results_raw = results_raw[results_raw[PROBLEM_TYPE].isin(problem_type)]
            else:
                results_raw = results_raw[results_raw[PROBLEM_TYPE] == problem_type]
            print(f"Filtering to the following problem_type: {problem_type}")
        if banned_datasets is not None:
            results_raw = results_raw[~results_raw[DATASET].isin(banned_datasets)]
        if valid_datasets is not None:
            results_raw = results_raw[results_raw[DATASET].isin(valid_datasets)]
        if infer_batch_size is not None:
            results_raw = self._update_infer_batch_size(results_raw=results_raw, infer_batch_size=infer_batch_size)
        if self._framework_nan_fill is not None:
            results_raw = fill_missing_results_with_default(
                framework_nan_fill=self._framework_nan_fill, frameworks_to_fill=frameworks, results_raw=results_raw
            )
        elif self._worst_nan_fill:
            results_raw = fill_missing_results_with_worst(
                frameworks_to_consider=frameworks, frameworks_to_fill=frameworks, results_raw=results_raw
            )

        if frameworks is not None:
            results_raw = self._filter_frameworks(results_raw=results_raw, frameworks=frameworks)
        if treat_folds_as_datasets:
            results_raw = convert_folds_into_separate_datasets(results_raw=results_raw)
            folds = [0]
        if self._filter_errors:
            results_raw = self.filter_errors(results_raw=results_raw, folds=folds)
        if frameworks is not None:
            frameworks_present = list(results_raw[FRAMEWORK].unique())
            if set(frameworks) != set(frameworks_present):
                diff = list(set(frameworks).symmetric_difference(set(frameworks_present)))
                diff = sorted(diff)
                raise AssertionError(f"Difference in expected frameworks present: {diff}")
        # Round error
        if self.round_error_decimals is not None:
            results_raw.loc[:, METRIC_ERROR] = results_raw[METRIC_ERROR].round(decimals=self.round_error_decimals)

        if self._columns_to_keep:
            results_raw = results_raw[self._columns_to_keep]
        return results_raw

    def task_metadata(self) -> pd.DataFrame | None:
        if isinstance(self._task_metadata, pd.DataFrame):
            return self._task_metadata
        elif self._task_metadata is not None:
            return load_task_metadata(path=self._task_metadata)
        else:
            return None

    def _clean_data(self, results_raw):
        assert self._task_metadata is not None, (
            f"Cannot clean data if task_metadata is None! "
            f"Either set `clean_data=False` or specify `task_metadata` during init."
        )
        task_metadata = self.task_metadata()
        task_metadata[DATASET] = task_metadata["name"]
        # FIXME: TEMP
        results_raw = results_raw.drop(columns=[DATASET])
        results_raw["tid"] = results_raw["tid"].astype(int)
        pre_unique_tid = len(results_raw["tid"].unique())
        # results_raw['dataset'] = results_raw['dataset'].map({'numerai28_6': 'numerai28.6'}).fillna(results_raw['dataset'])
        results_raw = results_raw.merge(task_metadata[["NumberOfInstances", DATASET, "tid"]], on="tid")

        post_unique_tid = len(results_raw["tid"].unique())

        print(f"Joined with task_metadata, filtered task IDs: {pre_unique_tid} -> {post_unique_tid}")

        # FIXME: TEMP
        if self.convert_infer_time_to_per_row:
            results_raw[TIME_INFER_S] = results_raw[TIME_INFER_S] / results_raw["NumberOfInstances"] * 10
        return results_raw

    def _update_infer_batch_size(self, results_raw: pd.DataFrame, infer_batch_size: int):
        # Update infer time
        if f"pred_time_test_with_transform_batch_size_{infer_batch_size}" in results_raw.columns:
            # AutoGluon specific infer time column
            results_raw["time_infer_s"] = results_raw[
                f"pred_time_test_with_transform_batch_size_{infer_batch_size}"
            ].fillna(results_raw["time_infer_s"])
        if f"pred_time_test_with_transform_{infer_batch_size}" in results_raw.columns:
            # AutoGluon specific infer time column
            results_raw["time_infer_s"] = results_raw[f"pred_time_test_with_transform_{infer_batch_size}"].fillna(
                results_raw["time_infer_s"]
            )
        if f"infer_batch_size_df_{infer_batch_size}" in results_raw.columns:
            # The new infer time column in AMLB Nov 2023 results
            if f"infer_batch_size_file_{infer_batch_size}" in results_raw.columns:
                results_raw[f"infer_batch_size_df_{infer_batch_size}"] = results_raw[
                    f"infer_batch_size_df_{infer_batch_size}"
                ].fillna(results_raw[f"infer_batch_size_file_{infer_batch_size}"])
            results_raw["time_infer_s"] = results_raw[f"infer_batch_size_df_{infer_batch_size}"].fillna(
                results_raw["time_infer_s"]
            )
            # FIXME: Divide by infer_batch_size?
            if infer_batch_size != 1:
                raise AssertionError(
                    f"Debug this logic before working with batch_sizes other than 1 to make sure it is working as intended."
                )
        elif f"infer_batch_size_file_{infer_batch_size}" in results_raw.columns:
            # The new infer time column in AMLB Nov 2023 results
            tmp = results_raw[f"infer_batch_size_file_{infer_batch_size}"] / infer_batch_size
            results_raw["time_infer_s"] = tmp.fillna(results_raw["time_infer_s"])
        return results_raw

    def filter_errors(self, results_raw: pd.DataFrame, folds, frameworks: list = None):
        """
        For each framework in frameworks, ensure that only datasets without failures from this framework are kept.
        """
        # FIXME: Ensure correct folds, not just count
        if frameworks is None:
            frameworks = list(results_raw["framework"].unique())
        for f in frameworks:
            datasets_keep = results_raw[results_raw["framework"] == f]["dataset"].value_counts()
            datasets_keep = list(datasets_keep[datasets_keep == len(folds)].index)
            results_raw = results_raw[results_raw["dataset"].isin(datasets_keep)]
        return results_raw

    def _filter_frameworks(self, results_raw: pd.DataFrame, frameworks: list):
        return results_raw[results_raw["framework"].isin(frameworks)]

    def filter_datasets(
        self, *, max_rows=None, min_rows=None, max_rows_missing_val=None, max_features_categorical=None
    ):
        # TODO: Consider having task_metadata be its own class
        task_metadata = self.task_metadata()

        if max_rows is not None:
            task_metadata = task_metadata[task_metadata["NumberOfInstances"] <= max_rows]
        if min_rows is not None:
            task_metadata = task_metadata[task_metadata["NumberOfInstances"] >= min_rows]
        if max_rows_missing_val is not None:
            task_metadata = task_metadata[task_metadata["NumberOfInstancesWithMissingValues"] <= max_rows_missing_val]
        if max_features_categorical is not None:
            task_metadata = task_metadata[task_metadata["NumberOfSymbolicFeatures"] <= max_features_categorical]

        return list(task_metadata["name"])

    @staticmethod
    def _assert_unique_metric(results_raw: pd.DataFrame):
        """
        Assert that each dataset uses exactly one unique metric.

        Parameters
        ----------
        results_raw

        """
        dataset_metric_counts = results_raw[[DATASET, METRIC]].value_counts().reset_index(drop=False)
        num_metrics_per_dataset = dataset_metric_counts[DATASET].value_counts()
        dataset_metric_counts["metrics_in_dataset"] = dataset_metric_counts[DATASET].map(num_metrics_per_dataset)
        dataset_metric_counts = dataset_metric_counts[dataset_metric_counts["metrics_in_dataset"] > 1]
        if len(dataset_metric_counts) != 0:
            num_invalid_datasets = len(dataset_metric_counts[DATASET].unique())
            raise AssertionError(
                f"Found {num_invalid_datasets} datasets using multiple different metrics. This is not yet supported in BenchmarkEvaluator. "
                f"Each dataset must map to a single metric. "
                f"One way to avoid this error is to append the metric name to the dataset name."
                f"\nList of invalid datasets:\n"
                f"{dataset_metric_counts}"
            )



def f1_score(y_true, y_pred):
    assert len(y_true) == len(y_pred)

    TP = ((y_true == 1) & (y_pred == 1)).sum()
    FP = ((y_true == 0) & (y_pred == 1)).sum()
    FN = ((y_true == 1) & (y_pred == 0)).sum()

    precision = TP / (TP + FP) if TP + FP != 0 else 0
    recall = TP / (TP + FN) if TP + FN != 0 else 0

    return 2 * precision * recall / (precision + recall) if precision + recall != 0 else 0



import logging

from autogluon.bench.eval.benchmark_context.output_suite_context import OutputSuiteContext
from autogluon.common.savers import save_pd

logger = logging.getLogger(__name__)


def aggregate_results(s3_bucket, s3_prefix, version_name, constraint, include_infer_speed=False, mode="ray"):
    contains = f".{constraint}." if constraint else None
    result_path = f"{s3_prefix}{version_name}/"
    path_prefix = f"s3://{s3_bucket}/{result_path}"

    aggregated_results_name = f"results_automlbenchmark_{constraint}_{version_name}.csv"

    output_suite_context = OutputSuiteContext(
        path=path_prefix,
        contains=contains,
        include_infer_speed=include_infer_speed,
        mode=mode,
    )
    results_df = output_suite_context.aggregate_results()

    print(results_df)

    save_path = f"s3://{s3_bucket}/aggregated/{result_path}{aggregated_results_name}"
    save_pd.save(path=save_path, df=results_df)
    logger.info(f"Aggregated results saved to {save_path}!")



import zipfile
from io import BytesIO, TextIOWrapper
from typing import Optional, Set, Union

import boto3
import numpy as np
import pandas as pd

from autogluon.common.loaders import load_pd, load_pkl
from autogluon.common.utils.s3_utils import s3_path_to_bucket_prefix


class OutputContext:
    def __init__(self, path):
        """
        Parameters
        ----------
        path : str
            The S3 path to the output folder of an AMLB task
        """
        self._path = path

    @classmethod
    def from_results_path(cls, path):
        results_suffix = "/scores/results.csv"
        assert path.endswith(results_suffix)
        dataset_directory = path.rsplit(results_suffix, 2)[0] + "/"
        return cls(path=dataset_directory)

    @property
    def path(self):
        return self._path

    def get_s3_bucket(self, path=None):
        if path is None:
            path = self.path
        return s3_path_to_bucket_prefix(path)[0]

    def get_s3_prefix(self, path=None):
        if path is None:
            path = self.path
        return s3_path_to_bucket_prefix(path)[1]

    @property
    def path_results(self):
        return self.path + "scores/results.csv"

    @property
    def path_leaderboard(self):
        return self.path + "leaderboard.csv"

    @property
    def path_model_failures(self):
        return self.path + "model_failures.csv"

    @property
    def path_infer_speed(self):
        return self.path + "infer_speed.csv"

    @property
    def path_logs(self):
        return self.path + "logs.zip"

    @property
    def path_info(self):
        return self.path + "info/info.pkl"

    @property
    def path_info_file_sizes(self):
        return self.path + "info/file_sizes.csv"

    @property
    def path_zeroshot_metadata(self):
        return self.path + "zeroshot/zeroshot_metadata.pkl"

    def load_results(
        self,
        include_infer_speed: bool = False,
        keep_params: bool = True,
        allowed_tids: Optional[Set[int]] = None,
    ) -> Optional[pd.DataFrame]:
        """
        allowed_tids can be specified to ensure results are only returned if they belong to a specific set of tasks.
        """
        results = load_pd.load(self.path_results)
        if allowed_tids is not None:
            tid = self.get_tid(results_df=results)
            if tid not in allowed_tids:
                return None
        if not keep_params:
            results = results.drop(columns=["params"])
        if include_infer_speed:
            try:
                results = self._merge_results_with_infer_speed(results=results, infer_speed=None)
            except FileNotFoundError:
                return None
        return results

    def load_info_file_sizes(self) -> pd.DataFrame:
        return load_pd.load(self.path_info_file_sizes)

    def load_leaderboard(self) -> pd.DataFrame:
        return load_pd.load(self.path_leaderboard)

    def load_model_failures(self) -> pd.DataFrame:
        """Load and return the raw model failures file"""
        return load_pd.load(self.path_model_failures)

    def load_infer_speed(self) -> pd.DataFrame:
        return load_pd.load(self.path_infer_speed)

    def get_zeroshot_metadata_size_bytes(self) -> int:
        s3_bucket = self.get_s3_bucket()
        s3_prefix = self.get_s3_prefix(path=self.path_zeroshot_metadata)
        s3 = boto3.client("s3")
        response = s3.head_object(Bucket=s3_bucket, Key=s3_prefix)
        size = response["ContentLength"]
        return size

    def load_zeroshot_metadata(self, max_size_mb: float = None) -> dict:
        size = self.get_zeroshot_metadata_size_bytes()

        size_og_mb = round(size / 1e6, 3)

        if max_size_mb is not None and size_og_mb > max_size_mb:
            print(f"exit: {size_og_mb} | {self.path}")
            return None

        a = load_pkl.load(self.path_zeroshot_metadata)
        print(f"CUR Size: {size_og_mb} MB | {self.path}")
        return a

    def get_tid(self, results_df: pd.DataFrame = None) -> int:
        """
        Get OpenML task ID (tid)
        """
        if results_df is None:
            results_df = self.load_results()
        tid = int(results_df["id"].iloc[0].rsplit("/", 1)[-1])
        return tid

    def get_amlb_info(self, results_df: pd.DataFrame = None) -> str:
        if results_df is None:
            results_df = self.load_results()
        amlb_info = results_df["info"].iloc[0]
        if not isinstance(amlb_info, str) and np.isnan(amlb_info):
            amlb_info = None
        return amlb_info

    def get_single_leaderboard(self, columns_to_keep, with_infer_speed, i, num_contexts):
        print_msg = f"{i + 1}/{num_contexts} | {self.path}"
        scores = self.load_results()
        amlb_info: str = self.get_amlb_info(results_df=scores)
        if amlb_info is not None:
            print_msg = f"{print_msg}\n" f"\tAMLB_INFO: {amlb_info}"
        try:
            leaderboard = self.load_leaderboard()
            leaderboard = leaderboard.drop(columns=["features"], errors="ignore")
            if with_infer_speed:
                leaderboard = self._merge_leaderboard_with_infer_speed(leaderboard=leaderboard)
        except Exception as e:
            print(f"FAILURE: {print_msg}\n" f"\t{e.__class__.__name__}: {e}")
            return None
        else:
            result_val = scores.iloc[0]["result"]
            if np.isnan(result_val):
                print(
                    f"FAILURE (STRANGE!): {print_msg}\n"
                    f"\tPRIORITIZE DEBUGGING: This is a strange error!\n"
                    f"\t\tDespite leaderboard.csv existing, the overall result is NaN, indicating a failure. "
                    f"This is likely a bug in the dataset metadata, such as columns being misaligned, "
                    f"and not a bug with AutoGluon specifically."
                )
                return None

            # scores = scores[scores['fold'] == 0]
            # print(scores)
            scores = scores[columns_to_keep]
            scores = scores.rename(columns={"framework": "framework_parent"})

            # best_compressed = leaderboard[leaderboard['model'].str.contains('_FULL')]
            # best_distilled = leaderboard[leaderboard['model'].str.contains('_d1')].sort_values('score_val', ascending=False).head(1)
            best_weighted = (
                leaderboard[leaderboard["model"].str.contains("WeightedEnsemble_")]
                .sort_values("score_val", ascending=False)
                .head(1)
            )
            best_nonweighted = (
                leaderboard[~leaderboard["model"].str.contains("WeightedEnsemble_")]
                .sort_values("score_val", ascending=False)
                .head(1)
            )

            # best_compressed['model'] = 'autogluon_compressed'
            # best_distilled['model'] = 'autogluon_distilled'
            # FIXME: Doesn't work for refit_full!!! score_val is NaN!
            best_weighted["model"] = "autogluon_ensemble"
            best_nonweighted["model"] = "autogluon_single"
            # print(best_compressed)
            # print(best_distilled)
            # print(best_weighted)

            combined = pd.concat(
                [
                    leaderboard,
                    best_weighted,
                    # best_compressed,
                    # best_distilled,
                    # best_nonweighted,
                ],
                ignore_index=True,
            )
            # combined = combined.sort_values('score_test', ascending=False).reset_index(drop=True)
            combined["id"] = scores["id"][0]
            # print(combined)

            combined_full = pd.merge(combined, scores, on="id", how="left")

            combined_full["framework"] = combined_full["model"]
            # combined_full['result'] = combined_full['score_test']
            combined_full["duration"] = combined_full["fit_time"]
            # combined_full['predict_duration'] = combined_full['pred_time_test']

            # print(combined_full)
            print(f"SUCCESS: {print_msg}")
            return combined_full

    def get_model_failures(self) -> Union[pd.DataFrame, None]:
        """
        Load and return the model failures CSV as a pandas DataFrame if it exists, else return None.

        Will merge with the results to get additional information, akin to the leaderboard output.
        """
        results = self.load_results()
        try:
            model_failures_df = self.load_model_failures()
        except Exception as e:
            print(f"FAILURE:\n" f"\t{e.__class__.__name__}: {e}")
            return None
        else:
            results = results.rename(columns={"framework": "framework_parent"})
            model_failures_df["id"] = results["id"][0]
            model_failures_full_df = pd.merge(model_failures_df, results, on="id", how="left")
            return model_failures_full_df

    def get_info_file_sizes(self, sum: bool = False) -> Union[pd.DataFrame, None]:
        """
        Load and return the model info file sizes CSV as a pandas DataFrame if it exists, else return None.

        Will merge with the results to get additional information, akin to the leaderboard output.
        """
        results = self.load_results()
        results = results[["id", "task", "framework", "constraint", "fold", "type", "result", "metric"]]
        try:
            info_file_sizes = self.load_info_file_sizes()
        except Exception as e:
            print(f"FAILURE:\n" f"\t{e.__class__.__name__}: {e}")
            return None
        else:
            if sum:
                total_size = info_file_sizes["size"].sum()
                results["size"] = total_size
                return results
            else:
                info_file_sizes["id"] = results["id"][0]
                info_file_sizes_full_df = pd.merge(info_file_sizes, results, on="id", how="left")
                return info_file_sizes_full_df

    def get_logs(self) -> str:
        s3_bucket, s3_prefix = s3_path_to_bucket_prefix(s3_path=self.path_logs)
        s3 = boto3.client("s3", use_ssl=False)
        buffer = BytesIO(s3.get_object(Bucket=s3_bucket, Key=s3_prefix)["Body"].read())
        z = zipfile.ZipFile(buffer)
        for filename in z.namelist():
            if ".full.log" not in filename:
                continue
            raw_text = TextIOWrapper(z.open(filename)).read()
            return raw_text
        return ""

    def _merge_leaderboard_with_infer_speed(self, leaderboard: pd.DataFrame) -> pd.DataFrame:
        infer_speed_df = self.load_infer_speed()
        infer_speed_df = infer_speed_df[["model", "batch_size", "pred_time_test_with_transform"]]
        infer_speed_m_df = infer_speed_df.set_index(["model", "batch_size"], drop=True)
        a = infer_speed_m_df.to_dict()
        b = a["pred_time_test_with_transform"]
        c = dict()
        for key_pair, pred_time_test_with_transform in b.items():
            m = key_pair[0]
            bs = key_pair[1]
            col_name = f"pred_time_test_with_transform_{bs}"
            if col_name not in c:
                c[col_name] = {}
            c[col_name][m] = pred_time_test_with_transform
        c_df = pd.DataFrame(c).rename_axis("model").reset_index(drop=False)
        leaderboard = pd.merge(leaderboard, c_df, on="model")
        return leaderboard

    def _merge_results_with_infer_speed(self, results: pd.DataFrame, infer_speed: pd.DataFrame = None) -> pd.DataFrame:
        if infer_speed is None:
            infer_speed = self.load_infer_speed()
        infer_speed_best = infer_speed[infer_speed["model"] == "best"]
        batch_sizes = list(infer_speed_best["batch_size"].unique())
        for batch_size in batch_sizes:
            results[f"pred_time_test_with_transform_{batch_size}"] = infer_speed_best[
                infer_speed_best["batch_size"] == batch_size
            ].iloc[0]["pred_time_test_with_transform"]
        return results



import logging
import os
import subprocess
import sys

from autogluon.bench.frameworks.benchmark import Benchmark

logger = logging.getLogger(__name__)


class TimeSeriesBenchmark(Benchmark):
    def setup(
        self,
        git_uri: str = "https://github.com/openml/automlbenchmark.git",
        git_branch: str = "stable",
        framework: str = "AutoGluon:stable",
        user_dir: str = None,
    ):
        """Sets up the virtual environment for timeseries benchmark."""
        setup_script_path = os.path.join(os.path.abspath(os.path.dirname(__file__)), "setup.sh")
        command = [setup_script_path, git_uri, git_branch, self.benchmark_dir, framework]
        if user_dir is not None:
            command.append(user_dir)
        result = subprocess.run(command)
        if result.returncode != 0:
            logger.error(result.stderr)
            sys.exit(1)
        else:
            logger.info(result.stdout)
            logger.info("Successfully set up the environment under %s/.venv.", self.benchmark_dir)

    def run(
        self,
        framework: str,
        benchmark: str,
        constraint: str,
        task: str = None,
        fold: int = None,
        user_dir: str = None,
    ):
        """Runs the timeseries benchmark.

        Args:
            framework (str): The name of the framework to use. Examples: "AutoGluon:latest", "AutoGluon:stable".
            benchmark (str): The name of the benchmark to run.
            constraint (str): The name of the constraint to use.
            task (str): The name of the task to run (default: None).
            user_dir (str): Path to custom configs to use (default: None).

        Returns:
            None
        """
        if os.environ.get("RUNNING_IN_DOCKER", "false") == "true":
            venv_base_dir = os.environ["VENV_BASE_DIR"]
        else:
            venv_base_dir = self.benchmark_dir

        exec_script_path = os.path.join(os.path.abspath(os.path.dirname(__file__)), "exec.sh")
        command = [
            exec_script_path,
            framework,
            benchmark,
            constraint,
            venv_base_dir,
            self.metrics_dir,
        ]

        if task is not None:
            command += ["-t", task]

        if fold is not None:
            command += ["-f", str(fold)]

        if user_dir is not None:
            command += ["-u", user_dir]

        result = subprocess.run(command)
        if result.returncode != 0:
            sys.exit(1)
        else:
            logger.info(f"Benchmark {self.benchmark_name} is complete.")



from autogluon.bench.utils.general_utils import download_dir_from_s3, download_file_from_s3, upload_to_s3


def test_upload_to_s3(mocker, tmp_path):
    config_file = tmp_path / "config.yaml"
    config_file.touch()

    s3_client_mock = mocker.Mock()
    mocker.patch("boto3.client", return_value=s3_client_mock)

    bucket = "test_bucket"
    benchmark_name = "test_benchmark"

    s3_path = upload_to_s3(s3_bucket=bucket, s3_dir=benchmark_name, local_path=str(config_file))

    s3_client_mock.upload_file.assert_called_once_with(
        str(config_file),
        bucket,
        f"{benchmark_name}/config.yaml",
    )

    assert s3_path == f"s3://{bucket}/{benchmark_name}/config.yaml"


def test_download_file_from_s3(mocker, tmp_path):
    s3_client_mock = mocker.Mock()
    mocker.patch("boto3.client", return_value=s3_client_mock)

    s3_path = "s3://test_bucket/configs/test_file.txt"
    local_path = download_file_from_s3(s3_path, str(tmp_path))

    s3_client_mock.download_file.assert_called_once_with(
        "test_bucket",
        "configs/test_file.txt",
        str(tmp_path / "test_file.txt"),
    )

    assert local_path == str(tmp_path / "test_file.txt")


def test_download_dir_from_s3(mocker, tmp_path):
    s3_client_mock = mocker.Mock()
    mocker.patch("boto3.client", return_value=s3_client_mock)

    # Mock the response of list_objects
    mock_response = {
        "Contents": [
            {"Key": "configs/test_file1.txt"},
            {"Key": "configs/test_file2.txt"},
        ]
    }
    s3_client_mock.list_objects.return_value = mock_response

    s3_path = "s3://test_bucket/configs"
    local_path = download_dir_from_s3(s3_path, str(tmp_path))

    expected_calls = [
        (("test_bucket", "configs/test_file1.txt", str(tmp_path / "test_file1.txt")),),
        (("test_bucket", "configs/test_file2.txt", str(tmp_path / "test_file2.txt")),),
    ]
    assert s3_client_mock.download_file.call_args_list == expected_calls

    assert local_path == str(tmp_path)
